@@ -17,6 +17,7 @@
 
 // C-Includes
 #include <math.h>
+#include <limits.h>
 
 // QT-Includes
 #include <qevent.h>
@@ -29,7 +30,6 @@
 // Project-Includes
 #include "ksimdata.h"
 #include "compview.h"
-#include "component.h"
 #include "componentaddon.h"
 #include "compcontainer.h"
 #include "connectorbase.h"
@@ -39,9 +39,11 @@
 #include "ksimus.h"
 #include "ksimwidget.h"
 #include "ksimeditor.h"
+#include "ksimusview.h"
 #include "ksimundo.h"
 #include "ksimdebug.h"
 #include "resource.h"
+#include "module.h"
 
 // Forward declaration
 
@@ -149,18 +151,18 @@ static void installGlobalEventFilter(QWidget * main, QWidget * eventSource)
 	
 	const QObjectList * list = main->children();
 	
-    if (list)
-    {
+	if (list)
+	{
 		QObject * obj;
-	    for ( obj = ((QObjectList*)list)->first(); obj; obj = ((QObjectList*)list)->next() )
-    	{
-    		if (obj->isWidgetType())
-	    	{
+		for ( obj = ((QObjectList*)list)->first(); obj; obj = ((QObjectList*)list)->next() )
+		{
+			if (obj->isWidgetType())
+			{
 				installGlobalEventFilter((QWidget*)obj, eventSource);
-	    	}
-	    }
-    }
-}	
+			}
+		}
+	}
+}
 
 static void setGlobalMouseTracking(QWidget * main, bool tracking)
 {
@@ -184,7 +186,7 @@ static void setGlobalMouseTracking(QWidget * main, bool tracking)
 	}
 }	
 
-ConnectorBase * CompView::lastHitConnector;
+ConnectorBase * CompView::lastHitConnector = 0;
 
 
 //############################################################################
@@ -210,9 +212,7 @@ public:
 	};
 	~CompViewPrivate()
 	{
-		if (widgetList)
-			delete widgetList;
-		
+		delete widgetList;
 	};
 	
 	/** Position and dimension of the component view */
@@ -239,6 +239,7 @@ CompView::CompView(Component * comp, eViewType viewType)
 		m_viewChanged(false)
 {
 	m_p = new CompViewPrivate(viewType);
+	CHECK_PTR(m_p);
 	
 	switch(viewType)
 	{
@@ -305,38 +306,13 @@ void CompView::mouseRelease(QMouseEvent *, QPainter *)
 /** New position of the comoponent view */
 void CompView::setPos(const QPoint & pos)
 {
-	QPoint newPos(pos);
-	
-	if (getComponent()->getContainer()->isVisible())
-	{	
-		// Limit position
-		QSize mapSize;
-		if (getViewType() == SHEET_VIEW)
-		{
-			mapSize = getComponent()->getContainer()->getSheetSize();
-		}
-		else
-		{ 	
-			mapSize = getComponent()->getContainer()->getUserSize();
-		}
+	QSize mapSize(  (getViewType() == SHEET_VIEW)
+	              ?  getComponent()->getContainer()->getSheetSize()
+	              :  getComponent()->getContainer()->getUserSize()  );
 
-		if (newPos.x() > (mapSize.width() - getPlace().width()))
-		{
-			newPos.setX(mapSize.width() - getPlace().width());
-		}
-		if (newPos.y() > (mapSize.height() - getPlace().height()))
-		{
-			newPos.setY(mapSize.height() - getPlace().height());
-		}
-		if (newPos.x() < 0)
-		{
-			newPos.setX(0);
-		}
-		if (newPos.y() < 0)
-		{
-			newPos.setY(0);
-		}
-	}
+	// Limit position
+	QPoint newPos(QMAX(0, QMIN(pos.x(), mapSize.width()  - getPlace().width())),
+	              QMAX(0, QMIN(pos.y(), mapSize.height() - getPlace().height())));
 	
 	// now map to grid
 	if (isGridSnapEnabled())
@@ -346,14 +322,25 @@ void CompView::setPos(const QPoint & pos)
 	
 	if(m_p->place.topLeft() != newPos)
 	{
-		// Remove Object from sheet map
-		updateSheetMap(false);
-		m_p->place.moveTopLeft(newPos);
-		// Insert Object to sheet map
-		updateSheetMap(true);
-
-		emit signalMove(getPlace().topLeft());
-		emit signalMoveWidget(getWidgetPlace().topLeft());
+		if (getComponent()->getContainer()->isVisible() && !isHidden())
+		{
+			// Remove Object from sheet map
+			updateSheetMap(false);
+			
+			// Store new position
+			m_p->place.moveTopLeft(newPos);
+			
+			// Insert Object to sheet map
+			updateSheetMap(true);
+			
+			emit signalMove(getPlace().topLeft());
+			emit signalMoveWidget(getWidgetPlace().topLeft());
+		}
+		else
+		{
+			// Store new position
+			m_p->place.moveTopLeft(newPos);
+		}
 	}
 }
 
@@ -371,31 +358,27 @@ void CompView::setPlace(const QRect & place, bool degree0)
 		newPlace.setSize(convertSize(newPlace.size(), ROT_0_DEG, convertRotToInt(getRotation())));
 	}
 	
-	
-	if (getComponent()->getContainer()->isVisible())
-	{	
-		// Limit position
-		QSize mapSize;
-		if (getViewType() == SHEET_VIEW)
-		{
-			mapSize = getComponent()->getContainer()->getSheetSize();
-		}
-		else
-		{
-			mapSize = getComponent()->getContainer()->getUserSize();
-		}
-		if (newPlace.bottom() > mapSize.height())
-			newPlace.moveBy(0, mapSize.height() - newPlace.bottom());
-		
-		if (newPlace.right() > mapSize.width())
-			newPlace.moveBy(mapSize.width() - newPlace.right(), 0);
-		
-		if (newPlace.top() < 0)
-			newPlace.moveBy(0, - newPlace.top());
-		
-		if (newPlace.left() < 0)
-			newPlace.moveBy( -newPlace.left(), 0);
+	// Limit position
+	QSize mapSize;
+	if (getViewType() == SHEET_VIEW)
+	{
+		mapSize = getComponent()->getContainer()->getSheetSize();
 	}
+	else
+	{
+		mapSize = getComponent()->getContainer()->getUserSize();
+	}
+	if (newPlace.bottom() > mapSize.height())
+		newPlace.moveBy(0, mapSize.height() - newPlace.bottom());
+	
+	if (newPlace.right() > mapSize.width())
+		newPlace.moveBy(mapSize.width() - newPlace.right(), 0);
+		
+	if (newPlace.top() < 0)
+		newPlace.moveBy(0, - newPlace.top());
+		
+	if (newPlace.left() < 0)
+		newPlace.moveBy( -newPlace.left(), 0);
 	
 	if (isGridSnapEnabled())
 	{
@@ -404,19 +387,27 @@ void CompView::setPlace(const QRect & place, bool degree0)
 	
 	if (newPlace != m_p->place)
 	{
-		// Remove Object from sheet map
-		updateSheetMap(false);
-		m_p->place = newPlace;
-		// Insert Object to sheet map
-		updateSheetMap(true);
+		if (getComponent()->getContainer()->isVisible())
+		{
+			// Remove Object from sheet map
+			updateSheetMap(false);
+			m_p->place = newPlace;
+			// Insert Object to sheet map
+			updateSheetMap(true);
 
-		// Don't forget setRotation !!
-		emit signalMove(getPlace().topLeft());
-		emit signalResize(getPlace().size());
-		emit signalMoveWidget(getWidgetPlace().topLeft());
-		emit signalResizeWidget(getWidgetPlace().size());
+			// Don't forget setRotation !!
+			emit signalMove(getPlace().topLeft());
+			emit signalResize(getPlace().size());
+			emit signalMoveWidget(getWidgetPlace().topLeft());
+			emit signalResizeWidget(getWidgetPlace().size());
 	
-		resize();
+			resize();
+			
+		}
+		else
+		{
+			m_p->place = newPlace;
+		}
 	}
 }
 
@@ -547,9 +538,9 @@ eHitType CompView::isHit(int x, int y) const
 			p = mapFromRotation(p);
 		}
 		
-		for (	conn = getComponent()->getConnList()->first();
-				conn;
-				conn = getComponent()->getConnList()->next())
+		for (conn = getComponent()->getConnList()->first();
+		     conn;
+		     conn = getComponent()->getConnList()->next())
 		{
 			if (CONNECTOR_HIT == conn->isHit(p.x(),p.y()))
 			{
@@ -667,7 +658,7 @@ ConnectorBase * CompView::getLastHitConnector() const
 if insert = true, delete compview to sheet map */
 void CompView::updateSheetMap(bool insert)
 {
-	if (getComponent()->getContainer()->isVisible())
+	if (getComponent()->getContainer()->isVisible() && !isHidden())
 	{
 		QRect mapPlace;
 	
@@ -677,10 +668,30 @@ void CompView::updateSheetMap(bool insert)
 		mapPlace.setBottom(getPlace().bottom() / gridY);
 		
 		getComponentMap()->addPlace(mapPlace, insert
-									?  getComponentMap()->costComponent
-									: -getComponentMap()->costComponent );
+		                                      ?  getComponentMap()->costComponent
+		                                      : -getComponentMap()->costComponent );
 	}
 }
+
+void CompView::setViewChanged(bool changed)
+{
+	m_viewChanged = changed;
+	if (changed && getComponent()->getContainer()->isParentComponent())
+	{
+		// Module !!!
+		Module * module((Module *)getComponent()->getContainer()->getParentComponent());
+		if (module->getSheetView())
+		{
+			module->getSheetView()->setViewChanged(true);
+		}
+		if (module->getUserView())
+		{
+			module->getUserView()->setViewChanged(true);
+		}
+	}
+};
+
+
 /** Creates a new Widget  */
 QWidget * CompView::createCompViewWidget(QWidget *)
 {
@@ -798,16 +809,28 @@ void CompView::setHide(bool hide)
 	{
 		if(hide)
 		{
-			m_p->flags |= FLAGS_HIDDEN;
-			// Remove object from sheet map
+			// Remove object from sheet map (before set hidden!)
 			updateSheetMap(false);
+			// set hidden;
+			m_p->flags |= FLAGS_HIDDEN;
+			// Unselect view
+			for(QListIterator<KSimusView> it(*getComponent()->getDoc()->getViewList());it.current();++it)
+			{
+				KSimEditor * ed = it.current()->getEditor();
+				if (ed) ed->select(this, false);
+			}
 			emit signalHide();
 		}
 		else
 		{
 			m_p->flags &= ~FLAGS_HIDDEN;
-			// Insert object to sheet map
+			// Insert object to sheet map (after set not hidden)
 			updateSheetMap(true);
+			moveToBestPlace();
+			emit signalMove(getPlace().topLeft());
+			emit signalResize(getPlace().size());
+			emit signalMoveWidget(getWidgetPlace().topLeft());
+			emit signalResizeWidget(getWidgetPlace().size());
 			emit signalShow();
 		}
 	}
@@ -816,7 +839,12 @@ void CompView::setHide(bool hide)
 /** Toogle the hide flag */
 void CompView::slotToggleHide()
 {
+	KSimusDoc * doc = getComponent()->getDoc();
+	CHECK_PTR(doc);
+
+	doc->getUndo()->changeProperty(this, i18n("Hide User View"));
 	setHide(!isHidden());
+	doc->setModified();
 }
 
 /** Returns true, if component view is hidden */
@@ -929,6 +957,12 @@ bool CompView::isGridSnapEnabled() const
 	return m_p->flags & FLAGS_ENABLE_GRID_SNAP;
 }
 
+Component::eComponentType CompView::getComponentType() const
+{
+	return getComponent()->getComponentType();
+}
+
+	
 /** Returns a list of all widgets of the component */
 KSimWidgetList * CompView::getWidgetList()
 {
@@ -1267,13 +1301,6 @@ void CompViewSize::drawBound(QPainter * p)
 {
 	CompView::drawBound(p);
 	p->setRasterOp(NotROP);
-//	p->setPen(NoPen);
-//	p->setBrush(SolidPattern);
-	
-//	p->drawRect(getPlace().left(), getPlace().top(), HANDLE_SIZE, HANDLE_SIZE);
-//	p->drawRect(getPlace().left(), getPlace().bottom(), HANDLE_SIZE, -HANDLE_SIZE);
-//	p->drawRect(getPlace().right(), getPlace().top(), -HANDLE_SIZE, HANDLE_SIZE);
-//	p->drawRect(getPlace().right(), getPlace().bottom(), -HANDLE_SIZE, -HANDLE_SIZE);
 
   #define HANDLE_THICK 2
 	
@@ -1319,7 +1346,7 @@ eHitType CompViewSize::isHit(int x, int y) const
 	place.rTop() -= 1;
 	place.rBottom() += HANDLE_THICK;
 
-	if (place.contains(x,y))
+	if (place.contains(x,y) && !isHidden())
 	{
 		if (((x <= place.left()  + HANDLE_SIZE) && (y <= place.top()    + HANDLE_SIZE)) //top left
 		 || ((x >= place.right() - HANDLE_SIZE) && (y >= place.bottom() - HANDLE_SIZE)))//bottom right
@@ -1378,7 +1405,7 @@ QRect CompViewList::getRect() const
 	FOR_EACH_COMPVIEW(it, *this)
 	{
 		// do not if wire
-		if (!it.current()->getComponent()->isWire())
+		if (!it.current()->getComponent()->isWire() && !it.current()->isHidden())
 		{
 			empty = false;
 			if (minX > it.current()->getPlace().left())
@@ -1397,5 +1424,41 @@ QRect CompViewList::getRect() const
 		return QRect(QPoint(minX,minY),QPoint(maxX,maxY));
 }
 
+
+void CompViewList::insertCompView(CompView * cv)
+{
+	Component::eComponentType type = cv->getComponentType();
+	unsigned int step = count() / 2;
+	unsigned int i = step;
+	
+	// Step fast to the position
+	while(step >= 2)
+	{
+		step /= 2;
+		if (type < at(i)->getComponentType())
+		{
+			i -= step;
+		}
+		else
+		{
+			i += step;
+		}
+	}
+	
+	while((i > 0) && (type < at(i)->getComponentType()))
+	{
+		i --;
+	}
+	
+	while((i < count()) && !(type < at(i)->getComponentType()))
+	{
+		i ++;
+	}
+	insert(i,cv);
+}
+
+
+
+
 //#############################################################################
 //#############################################################################
