@@ -28,6 +28,7 @@
 #include <qtooltip.h>
 #include <qbitmap.h>
 #include <qpaintdevicemetrics.h>
+#include <qcursor.h>
 
 // include files for KDE
 #include <kinstance.h>
@@ -57,6 +58,9 @@
 #include "ksimdebug.h"
 #include "componentpropertydialog.h"
 #include "packageinfo.h"
+#include "watchwidget.h"
+#include "wire.h"
+#include "wireproperty.h"
 
 
 // Scroll directions
@@ -83,7 +87,7 @@
 #include "wire.cur"
 #include "wire_mask.cur"
 
-class KSimEditorCursor
+class KSimEditor::Cursor
 {
 public:
 	enum KSimCursorType { arrowCursor, upArrowCursor, crossCursor, waitCursor, ibeamCursor,
@@ -91,8 +95,8 @@ public:
 	                      sizeAllCursor, blankCursor, splitVCursor, splitHCursor,
 	                      pointingHandCursor, forbiddenCursor, wireCursor };
 	
-	KSimEditorCursor(KSimEditor * editor);
-	~KSimEditorCursor() {};
+	Cursor(KSimEditor * editor);
+	~Cursor() {};
 	
 	void set(KSimCursorType newCursor);
 	void setNow(KSimCursorType newCursor);
@@ -103,10 +107,9 @@ private:
 	KSimCursorType m_current;
 	KSimEditor * m_editor;
 	QCursor * m_wireCursor;
-	
 };
 
-KSimEditorCursor::KSimEditorCursor(KSimEditor * editor)
+KSimEditor::Cursor::Cursor(KSimEditor * editor)
 	:	m_editor(editor)
 {
 	// Wire cursor
@@ -118,7 +121,7 @@ KSimEditorCursor::KSimEditorCursor(KSimEditor * editor)
 	setNow(arrowCursor);
 };
 
-void KSimEditorCursor::set(KSimCursorType newCursor)
+void KSimEditor::Cursor::set(KSimCursorType newCursor)
 {
 	if(get() != newCursor)
 	{
@@ -126,7 +129,7 @@ void KSimEditorCursor::set(KSimCursorType newCursor)
 	}
 }
 
-void KSimEditorCursor::setNow(KSimCursorType newCursor)
+void KSimEditor::Cursor::setNow(KSimCursorType newCursor)
 {
 	m_current = newCursor;
 	
@@ -177,24 +180,24 @@ KSimEditor::KSimEditor(QWidget *parent, const char *name)
 	
 	drawMap = new QPixmap;
 	CHECK_PTR(drawMap);
-  	
-	m_myCursor = new KSimEditorCursor(this);
+	
+	m_myCursor = new KSimEditor::Cursor(this);
 	CHECK_PTR(m_myCursor);
 	
 	setMouseTracking(true);
 	setEditorMode(EM_SELECT);
 	setEditorView(EV_SHEETVIEW);
 	lmbDown = dragging = isMapResized = false;
-   	
+	
 	selected.setAutoDelete(false);
-   	
+	
 	setBackgroundMode(NoBackground);
 
 	connect (&autoScrollTimer, SIGNAL(timeout()), SLOT(autoScroll()));
 	connect (&delayedStatusHelpMsgTimer, SIGNAL(timeout()), SLOT(slotDelayedStatusHelpMsg()));
 	
 	
-	QFont newFont("helvetica",12);
+	QFont newFont(QString::fromLatin1("helvetica"),12);
 	setFont(newFont);
 	
 }
@@ -232,10 +235,8 @@ void KSimEditor::print(KPrinter *pPrinter)
 	
 #define PRINT_FACTOR   2
 	
-	
-	
 	QPainter p;
-		p.begin(pPrinter);
+	p.begin(pPrinter);
 	
 /*	QPaintDeviceMetrics pdmEditor(this);
 	KSIMDEBUG("-------- this ---------");
@@ -248,15 +249,15 @@ void KSimEditor::print(KPrinter *pPrinter)
 
 	
 	QPaintDeviceMetrics pdmPrinter(pPrinter);
-	KSIMDEBUG("-------- pPrinter ---------");
+/*	KSIMDEBUG("-------- pPrinter ---------");
 	KSIMDEBUG_VAR("",pdmPrinter.width());
 	KSIMDEBUG_VAR("",pdmPrinter.widthMM());
 	KSIMDEBUG_VAR("",pdmPrinter.height());
 	KSIMDEBUG_VAR("",pdmPrinter.heightMM());
 	KSIMDEBUG_VAR("",pdmPrinter.logicalDpiX());
-	KSIMDEBUG_VAR("",pdmPrinter.logicalDpiY());
+	KSIMDEBUG_VAR("",pdmPrinter.logicalDpiY());*/
 	
-	QFont newFont("helvetica",12);
+	QFont newFont(QString::fromLatin1("helvetica"),12);
 	p.setFont(newFont);
 
 	int x = 0;
@@ -306,7 +307,7 @@ void KSimEditor::print(KPrinter *pPrinter)
 			}
 			else
 			{
-				register QPoint pos(it.current()->getPos());
+				QPoint pos(it.current()->getPos());
 				p.translate(pos.x(), pos.y());
 			}
 		
@@ -414,9 +415,10 @@ void KSimEditor::drawDragRect ()
 
 	switch (editorMode)
 	{
-		case EM_MOVE:	
-		case EM_MOVE_COPY:	
-		case EM_INSERT_MOVE:	
+		case EM_MOVE:
+		case EM_MOVE_COPY:
+		case EM_INSERT_MOVE:
+		case EM_PAST_MOVE:
 			FOR_EACH_COMPVIEW(it, selected)
 			{
 				QPoint oldPos = it.current()->getPos();
@@ -486,7 +488,7 @@ void KSimEditor::select (Component *comp, bool sel)
 void KSimEditor::unselectAll()
 {
 	while (!selected.isEmpty())
-		select (selected.first(), FALSE);
+		select (selected.first(), false);
 
 	emit deleteAllowed(false);
 	emit cutAllowed(false);
@@ -503,7 +505,7 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 	if (ev->button() == LeftButton)
 	{
 		dragStart = dragNow = mousePos;
-		lmbDown = TRUE;
+		lmbDown = true;
 		dragging = isMapResized = false;
 
 		switch (editorMode)
@@ -632,10 +634,15 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 			{
 				Component * newComp;
 				CompView * newCompView;
-				QRect place;
-				QPoint pos;
-				if (g_library->getComponentLib()->createComponent(&newComp, getDoc()->getContainer(), m_insertCI->getLibName()))
+				if (g_library->getComponentLib()->createComponent(&newComp, getContainer(), m_insertCI->getLibName()))
 				{
+					KSimUndo * undo = getDoc()->getUndo();
+					undo->begin(i18n("New Component"));
+					getContainer()->addComponent(newComp);
+					undo->end();
+					getDoc()->setModified();
+					setEditorMode(EM_INSERT_MOVE);
+
 					if (newComp->isModule())
 					{
 						if (((Module*)newComp)->getModuleFile().isEmpty())
@@ -644,36 +651,29 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 							ASSERT(!getApp()->getModuleFile().isEmpty());
 						
 							const ModuleInfo * mi =	ModuleData::makeModuleInfo(getApp()->getModuleFile());
-              if (!mi)
-              {
-              	// No valid module
-           			KMessageBox::error(0,i18n("The file %1 contains no valid module!").arg(getApp()->getModuleFile()),
-           													i18n("Open Module"));
+							if (!mi)
+							{
+								// No valid module
+								KMessageBox::error(0,i18n("The file %1 contains no valid module!").arg(getApp()->getModuleFile()),
+								                     i18n("Open Module"));
 
-              	delete newComp;
-              	setEditorMode(EM_SELECT);
-              	break;
-              }
+//								delete newComp;
+								getContainer()->delComponent(newComp);
+								undo->hiddenUndo();  // Remove last action from undo
+								setEditorMode(EM_SELECT);
+								lmbDown = false;
+								break;
+							}
 							((ComponentLibrary*)g_library->getComponentLib())->insert(mi, 0);
 							((Module*)newComp)->setModuleFile(getApp()->getModuleFile());
 						}
 					}
 					
-					KSimUndo * undo = getDoc()->getUndo();
-					undo->begin(i18n("New Component"));
-					getContainer()->addComponent(newComp);
-					undo->end();
-					getDoc()->setModified();
-					
-					
-					setEditorMode(EM_INSERT_MOVE);
 					newCompView = getSpecificCompView(newComp);
 					
 					// Position check
-					place = newCompView->getPlace();
-					pos.setX(dragStart.x() - place.width()/2);
-					pos.setY(dragStart.y() - place.height()/2);
-					pos = newCompView->mapToGrid(pos);
+					QRect place(newCompView->getPlace());
+					QPoint pos(newCompView->mapToGrid(QPoint(dragStart.x() - place.width()/2, dragStart.y() - place.height()/2)));
 					if (pos.x() < 0)
 						pos.setX(0);
 					if (pos.x() >= size.width() - place.width())
@@ -712,12 +712,12 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 				int minY = INT_MAX;
 				int maxX = INT_MIN;
 				int maxY = INT_MIN;
-				unselectAll();
-				getDoc()->setModified();
 				KSimUndo * undo = getDoc()->getUndo();
 				undo->begin(i18n("Copy Components"));
 				getContainer()->pastComponent(&compList, pos);
 				undo->end();
+				unselectAll();
+				getDoc()->setModified();
 				convertComponentToCompView(&compList, &selected);
 				// Calculate dimension and the middle of all components
 				FOR_EACH_COMPVIEW(it, selected)
@@ -750,9 +750,9 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 				select(selected.first(),true);
 				getContainer()->routeComponents();
 				refresh();
-				setEditorMode(EM_PAST_MOVE);
 
 				dragNow = mousePos;
+				setEditorMode(EM_PAST_MOVE);
 				mouseMoveEvent(ev);
 			}
 			break;
@@ -761,7 +761,7 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 				break;
 		}
 	}
-  	// Right mouse button
+	// Right mouse button
 	else if ((ev->button() == RightButton) && (!lmbDown))
 	{
 		if ((editorMode == EM_INSERT) || (editorMode == EM_PAST))
@@ -798,24 +798,36 @@ void KSimEditor::mousePressEvent (QMouseEvent *ev)
 					break;
 			}
 		}
+		else
+		{
+		
+			switch(getContainer()->isCompViewHit(mousePos, viewList))
+			{
+				case SPECIAL_HIT:
+					{
+						QPainter p(drawMap);
+						getContainer()->getFirstCompView()->mousePress(ev, &p);
+					}
+					break;
+				
+				default:
+					break;
+			}
+		}
 	}
 }
 
 static QString getComponentPartName(const Component * comp, const ConnectorBase * conn)
 {
-	QString s(i18n(comp->getInfo()->getName().latin1()));
-
-	if (conn)
+	if(conn)
 	{
-		s += ":" + conn->getName();
+		return conn->getFullName();
 	}
-		
-	if (comp->getName() != i18n(comp->getInfo()->getName().latin1()))	
+	if(comp)
 	{
-		s += " (" + comp->getName() + ")";
+		return comp->getName();
 	}
-	
-	return s;
+	return QString::null;
 }
 
 void KSimEditor::mouseMoveEvent (QMouseEvent *ev)
@@ -844,13 +856,14 @@ void KSimEditor::mouseMoveEvent (QMouseEvent *ev)
 			case SPECIAL_HIT:
 			case COMP_RESIZE_F_HIT:
 			case COMP_RESIZE_B_HIT:
-				delayedStatusHelpMsg(getComponentPartName(getContainer()->getFirstCompView()->getComponent(), 0));
+				delayedStatusHelpMsg(getComponentPartName(getContainer()->getFirstCompView()->getComponent(),
+				                                          (const ConnectorBase *) 0));
 				break;
 				
 			case CONNECTOR_HIT:
 			{
 				delayedStatusHelpMsg(getComponentPartName(getContainer()->getFirstCompView()->getComponent(),
-				                                      getContainer()->getFirstConnector()));
+				                                          getContainer()->getFirstConnector()));
 				break;
 			}
 		}
@@ -870,7 +883,7 @@ void KSimEditor::mouseMoveEvent (QMouseEvent *ev)
 			return;
 		}
 	
-		QRect sheet = QRect(QPoint(0,0), size);
+		QRect sheet(QPoint(0,0), size);
 		if ( dragging  ||
 			(abs(mousePos.x()-dragStart.x())>=4) ||
 			(abs(mousePos.y()-dragStart.y())>=4) )
@@ -1049,7 +1062,7 @@ void KSimEditor::mouseReleaseEvent (QMouseEvent *ev)
 	
 		if (dragging)
 			drawDragRect();
-        dragging = false;
+		dragging = false;
 
 		if ((editorMode == EM_MOVE) && (ev->state() & ControlButton))
 		{
@@ -1082,8 +1095,8 @@ void KSimEditor::mouseReleaseEvent (QMouseEvent *ev)
 				QRect selectArea = QRect (dragStart, dragNow).normalize();
 				FOR_EACH_COMPVIEW(it, *viewList)
 				{
- 					if (selectArea.contains (it.current()->getPlace()))
-						select (it.current(), TRUE);
+ 					if (selectArea.contains (it.current()->getPlace()) && !it.current()->isHidden())
+						select(it.current(), TRUE);
 				}
 				setEditorMode(EM_SELECT);
 				break;
@@ -1210,27 +1223,27 @@ void KSimEditor::setEditorCursor(QPoint * pMousePos, eHitType hit) const
 	{
 		case EM_INSERT:
 		case EM_PAST:
-				m_myCursor->set(KSimEditorCursor::crossCursor);
+				m_myCursor->set(KSimEditor::Cursor::crossCursor);
 			break;
 			
 		case EM_RESIZE_MAP:
 			if (   ((resizeDir ^ (RS_Left|RS_Up)) == 0)
 				|| ((resizeDir ^ (RS_Right|RS_Down)) == 0) )
-				m_myCursor->set(KSimEditorCursor::sizeFDiagCursor);
+				m_myCursor->set(KSimEditor::Cursor::sizeFDiagCursor);
 			else
-				m_myCursor->set(KSimEditorCursor::sizeBDiagCursor);
+				m_myCursor->set(KSimEditor::Cursor::sizeBDiagCursor);
 			break;
 			
 		case EM_COMP_RESIZE_B:
-				m_myCursor->set(KSimEditorCursor::sizeBDiagCursor);
+				m_myCursor->set(KSimEditor::Cursor::sizeBDiagCursor);
 			break;
 		
 		case EM_COMP_RESIZE_F:
-				m_myCursor->set(KSimEditorCursor::sizeFDiagCursor);
+				m_myCursor->set(KSimEditor::Cursor::sizeFDiagCursor);
 			break;
 		
 		case EM_WIRE:
-				m_myCursor->set(KSimEditorCursor::wireCursor);
+				m_myCursor->set(KSimEditor::Cursor::wireCursor);
 			break;
 		
 		default:
@@ -1250,12 +1263,12 @@ void KSimEditor::setEditorCursor(QPoint * pMousePos, eHitType hit) const
 					if ( ((myMousePos.x()<= RS_HANDLE_SIZE) && (myMousePos.y()<= RS_HANDLE_SIZE))
 					   ||((myMousePos.x()>= size.width()-RS_HANDLE_SIZE) && (myMousePos.y()>= size.height()-RS_HANDLE_SIZE)) )
 					{
-						m_myCursor->set(KSimEditorCursor::sizeFDiagCursor);
+						m_myCursor->set(KSimEditor::Cursor::sizeFDiagCursor);
 					}
 					else if ( ((myMousePos.x()<= RS_HANDLE_SIZE) && (myMousePos.y()>= size.height()-RS_HANDLE_SIZE))
 					        ||((myMousePos.x()>= size.width()-RS_HANDLE_SIZE) && (myMousePos.y()<= RS_HANDLE_SIZE)))
 					{
-						m_myCursor->set(KSimEditorCursor::sizeBDiagCursor);
+						m_myCursor->set(KSimEditor::Cursor::sizeBDiagCursor);
 					}
 					else
 					{
@@ -1269,27 +1282,27 @@ void KSimEditor::setEditorCursor(QPoint * pMousePos, eHitType hit) const
 							case NO_HIT:
 							case SPECIAL_HIT:
 							case NORMAL_HIT:
-								m_myCursor->set(KSimEditorCursor::arrowCursor);
+								m_myCursor->set(KSimEditor::Cursor::arrowCursor);
 								break;
 				
 							case CONNECTOR_HIT:
 							case WIRE_HIT:
-								m_myCursor->set(KSimEditorCursor::wireCursor);
+								m_myCursor->set(KSimEditor::Cursor::wireCursor);
 								break;
 					
 							case COMP_RESIZE_F_HIT:
-								m_myCursor->set(KSimEditorCursor::sizeFDiagCursor);
+								m_myCursor->set(KSimEditor::Cursor::sizeFDiagCursor);
 								break;
 					
 							case COMP_RESIZE_B_HIT:
-								m_myCursor->set(KSimEditorCursor::sizeBDiagCursor);
+								m_myCursor->set(KSimEditor::Cursor::sizeBDiagCursor);
 							break;
 						}
 					}
 				}
 				else
 				{
-					m_myCursor->set(KSimEditorCursor::arrowCursor);
+					m_myCursor->set(KSimEditor::Cursor::arrowCursor);
 				}
 				break;
 			}
@@ -1303,13 +1316,16 @@ void KSimEditor::setEditorCursor(QPoint * pMousePos, eHitType hit) const
 void KSimEditor::refreshSimMode(CompViewList * sheetList, CompViewList * userList)
 {
 	QPainter p (drawMap);
+	CompViewList cvList;
+	
 	if (EV_SHEETVIEW == editorView)
 	{
 		FOR_EACH_COMPVIEW(it, *sheetList)
 		{
 			if (it.current()->isViewChanged())
 			{
-				it.current()->draw(&p);
+				it.current()->setViewChanged(false);
+				cvList.append(it.current());
 			}
 		}
 	}
@@ -1319,10 +1335,12 @@ void KSimEditor::refreshSimMode(CompViewList * sheetList, CompViewList * userLis
 		{
 			if (it.current()->isViewChanged())
 			{
-				it.current()->draw(&p);
+				it.current()->setViewChanged(false);
+				cvList.append(it.current());
 			}
 		}
 	}
+	getContainer()->drawComponents(&p, &cvList);
 }
 /**  */
 bool KSimEditor::eventFilter( QObject * obj, QEvent * ev)
@@ -1344,7 +1362,8 @@ bool KSimEditor::eventFilter( QObject * obj, QEvent * ev)
 				                        mapFromGlobal(((QWidget*)obj)->mapToGlobal(((QMouseEvent*)ev)->pos())),
 				                        ((QMouseEvent*)ev)->button(),
 				                        ((QMouseEvent*)ev)->state() );
-				return event(&mouseEv);
+				/*return*/ event(&mouseEv);  // FIX for KDE3
+				return true;
 			}
 			break;
 			
@@ -1353,7 +1372,8 @@ bool KSimEditor::eventFilter( QObject * obj, QEvent * ev)
 			case QEvent::FocusIn:
 			case QEvent::FocusOut:
 			{
-				return event(ev);
+				/*return*/ event(ev);  // FIX for KDE3
+				return true;
 			}
 			break;
 			
@@ -1622,12 +1642,12 @@ void KSimEditor::backgroundPopup()
 	
 	ComponentPopupMenu * compMenu = new ComponentPopupMenu(menu);
 	connect(compMenu, SIGNAL(signalSelection(const ComponentInfo *)),
-			getApp(), SLOT(slotSelectCI(const ComponentInfo *)));
+	        getApp(), SLOT(slotSelectCI(const ComponentInfo *)));
 
-    getApp()->editDelete->plug(menu);
-    getApp()->editCut->plug(menu);
-    getApp()->editCopy->plug(menu);
-    getApp()->editPaste->plug(menu);
+	getApp()->editDelete->plug(menu);
+	getApp()->editCut->plug(menu);
+	getApp()->editCopy->plug(menu);
+	getApp()->editPaste->plug(menu);
 	
 	menu->insertSeparator();
 	menu->insertItem(i18n("New component"), compMenu, ID_COMP_MENU);
@@ -1647,6 +1667,7 @@ void KSimEditor::componentPopup(bool connectorHit)
 	int rot90Idx = 0;
 	int rot180Idx = 0;
 	int rot270Idx = 0;
+	int addWatchItemIdx = 0;
 	KSimUndo * undo = getDoc()->getUndo();
 	CHECK_PTR(undo);
 	QPopupMenu * menu = new QPopupMenu(0, "componentPopup");
@@ -1670,7 +1691,7 @@ void KSimEditor::componentPopup(bool connectorHit)
 	{
 		Component * comp = selected.first()->getComponent();
 		CompView * compView = getSpecificCompView(comp);
-		ConnectorBase * conn = 0;
+		ConnectorBase * conn = (ConnectorBase *)0;
 		
 		
 		// Init rotation
@@ -1711,6 +1732,8 @@ void KSimEditor::componentPopup(bool connectorHit)
 		{
 			conn = getContainer()->getFirstConnector();
 			conn->initPopupMenu(menu);
+			menu->insertSeparator();
+			addWatchItemIdx = menu->insertItem(i18n("&Add connector to watch view"));
 		}
 		
 		comp->initPopupMenu(menu);
@@ -1731,17 +1754,32 @@ void KSimEditor::componentPopup(bool connectorHit)
 		
 		if (res == idx)
 		{
-			ComponentPropertyDialog * dia;
-			dia = new ComponentPropertyDialog(comp);
-			comp->initPropertyDialog(dia);
-			dia->exec();
-			delete dia;
+			ComponentPropertyDialog::execute(comp, i18n("Component Properties"));
 		}
 		else if (res == connIdx)
 		{
-			ConnectorPropertyDialog * dia = new ConnectorPropertyDialog(comp->getConnList(), conn);
+			ConnectorPropertyDialog::execute(comp->getConnList(), conn, i18n("Connector Properties"));
+/*			ConnectorPropertyDialog * dia = new ConnectorPropertyDialog(comp->getConnList(), conn, i18n("Connector Properties"));
+
+			// Load last size
+			KConfig * config=kapp->config();
+			QString group(config->group());
+			config->setGroup("Connector/Property Dialog");
+			QSize size=config->readSizeEntry("Geometry");
+			config->setGroup(group);
+			if(!size.isEmpty())
+			{
+				dia->resize(size);
+			}
+	
 			dia->exec();
-			delete dia;
+
+			// Save size
+			config->setGroup("Connector/Property Dialog");
+			config->writeEntry("Geometry", dia->size());
+			config->setGroup(group);
+
+			delete dia;*/
 		}
 		else if (res == rot0Idx)
 		{
@@ -1785,10 +1823,12 @@ void KSimEditor::componentPopup(bool connectorHit)
 				kapp->invokeHelp(ci->getHTMLDescr(), pi->getInstance()->instanceName());
 			}
 		}
-		
-		
+		else if (res == addWatchItemIdx)
+		{
+			WatchItemBase * wi = conn->makeWatchItem();
+			getApp()->getWatchWidget()->addWatchItem(wi);
+		}
 
-		
 		// Not deleted ?
 		if (getContainer()->getComponentList()->find(comp) != -1)
 		{
@@ -1813,19 +1853,31 @@ void KSimEditor::wirePopup(const QPoint & pos)
 	CHECK_PTR(menu);
 	KSimUndo * undo = getDoc()->getUndo();
 	CHECK_PTR(undo);
-	Component * comp = getContainer()->getFirstCompView()->getComponent();
-	int res,idDelete, idTruncate, idProperty;
+	Wire * wire = (Wire*)getContainer()->getFirstCompView()->getComponent();
+	int res,idDelete, idTruncate, idProperty, addWatchItemIdx;
+	WireProperty * wp = wire->getWireProperty();
 
 	idDelete = menu->insertItem(i18n("&Delete wire"));
 	idTruncate = menu->insertItem(i18n("&Truncate wire"));
 	
-	if (comp->getConnList()->count() <= 2)
+	if (wire->getConnList()->count() <= 2)
 	{
 		// No truncate
 		menu->setItemEnabled(idTruncate, false);
 	}
 		
-	comp->initPopupMenu(menu);
+	wire->initPopupMenu(menu);
+
+	if (wp->inherits("WirePropertyInvalidBase"))
+	{
+		addWatchItemIdx = 0;
+	}
+	else
+	{
+		menu->insertSeparator();
+		addWatchItemIdx = menu->insertItem(i18n("&Add wire to watch view"));
+	}
+
 
 	menu->insertSeparator();
 	idProperty = menu->insertItem(i18n("P&roperties"));
@@ -1836,7 +1888,7 @@ void KSimEditor::wirePopup(const QPoint & pos)
 	if(res == idDelete)
 	{
 		undo->begin(i18n("Delete Wire"));
-		getContainer()->delComponent(comp);
+		getContainer()->delComponent(wire);
 		undo->end();
 		getDoc()->setModified();
 		getContainer()->routeComponents();
@@ -1845,7 +1897,7 @@ void KSimEditor::wirePopup(const QPoint & pos)
 	else if (res == idTruncate)
 	{
 		undo->begin(i18n("Truncate Wire"));
-		getContainer()->truncateWire((Wire*)comp, pos);
+		getContainer()->truncateWire(wire, pos);
 		undo->end();
 		getDoc()->setModified();
 		getContainer()->routeComponents();
@@ -1853,11 +1905,22 @@ void KSimEditor::wirePopup(const QPoint & pos)
 	}
 	else if (res == idProperty)
 	{
-		ComponentPropertyDialog * dia;
-		dia = new ComponentPropertyDialog(comp);
-		comp->initPropertyDialog(dia);
-		dia->exec();
-		delete dia;
+		ComponentPropertyDialog::execute(wire, i18n("Component Properties"));
+	}
+	else if (res == addWatchItemIdx)
+	{
+		WireProperty * wp = wire->getWireProperty();
+
+		if (wp)
+		{
+			getApp()->getWatchWidget()->addWatchItem(wp->makeWatchItem());
+		}
+		else
+		{
+			KMessageBox::sorry( (QWidget *)0,
+			                    i18n("You can't add the wire to the watch view until \n"
+			                         "the data type is not specified!\n"));
+		}
 	}
 	else if (res == -1)
 	{
@@ -1869,9 +1932,9 @@ void KSimEditor::wirePopup(const QPoint & pos)
 	}
 	
 	// Not deleted ?
-	if (getContainer()->getComponentList()->find(comp) != -1)
+	if (getContainer()->getComponentList()->find(wire) != -1)
 	{
-		comp->menuExecuted();
+		wire->menuExecuted();
 	}
 	
 	delete menu;
