@@ -23,6 +23,7 @@
 #include <qtabwidget.h>
 #include <qvbox.h>
 #include <qstring.h>
+#include <qtimer.h>
 
 // include files for KDE
 #include <kprinter.h>
@@ -34,6 +35,7 @@
 #include <kconfig.h>
 #include <kstdaction.h>
 #include <kstddirs.h>
+#include <kstatusbar.h>
 
 // application specific includes
 #include "globals.h"
@@ -54,9 +56,11 @@
 #include "moduledialog.h"
 #include "module.h"
 #include "loglist.h"
-#include "loglistitem.h"
+#include "loglistdialogwidget.h"
 #include "ksimpackagefilewidget.h"
+#include "ksimmodulefilewidget.h"
 #include "mapsizeswidget.h"
+#include "watchwidget.h"
 
 #define ID_STATUS_MSG 1
 #define ID_TIME_LABEL 2
@@ -71,14 +75,17 @@
 //################################################################################
 
 
-static EnumDict<eAppViewType> currentViewDict;
-
 EnumDict<eAppViewType>::tData EnumDict<eAppViewType>::data[]
-			= { {"Sheet View", APP_SHEET_VIEW},
-          {"User View",  APP_USER_VIEW},
-					{"Module View", APP_MODULE_VIEW},
-          {0,(eAppViewType)0}};
+      = { {"Sheet View",   APP_SHEET_VIEW},
+          {"User View",    APP_USER_VIEW},
+          {"Module View",  APP_MODULE_VIEW},
+          {0,             (eAppViewType)0}};
 
+static const EnumDict<eAppViewType> & getCurrentViewDict()
+{
+	static EnumDict<eAppViewType> currentViewDict;
+	return currentViewDict;
+}
 
 //################################################################################
 //################################################################################
@@ -108,6 +115,8 @@ public:
 	ModuleDialog * moduleDialog;
 	eAppViewType currentView;
 	QString moduleFile;
+	QTimer * messageTimer;
+	
 };
 
 
@@ -131,10 +140,38 @@ KSimusApp::KSimusApp(QWidget* , const char* name)
 {
 	bool loadLib = false;
 	
+	config=kapp->config();
+	
+	config->setGroup("General Options");
+	bool firstRead = config->readBoolEntry("First Start",true);
+	
+	if (firstRead)
+	{
+		// Some presets
+		QString group(config->group());
+	
+		// Default package dirs
+		config->setGroup("Packages");
+		KStandardDirs dirs;
+		QStringList packList = dirs.findDirs("lib",QString::fromLatin1(""));
+		unsigned int i;
+		for (i=0; i<packList.count(); i++)
+		{
+			packList[i] += QString::fromLatin1("ksimus/");
+		}
+		
+		config->writeEntry("Directories", packList);
+
+		config->setGroup(group);
+	}	
 	
 	m_p = new Private();
 	CHECK_PTR(m_p);
-	
+
+	m_p->messageTimer = new QTimer(this, "KSimusApp::messageTimer");
+	CHECK_PTR(m_p->messageTimer);
+	connect(m_p->messageTimer, SIGNAL(timeout()), this, SLOT(slotStatusMsgDefault()));
+                                                                                    	
 	if (!g_appList)
 	{
 		g_appList = new KSimusAppList();
@@ -144,7 +181,7 @@ KSimusApp::KSimusApp(QWidget* , const char* name)
 	
 //	KSIMDEBUG(QString("LanguageList: %1").arg(KGlobal::locale()->languageList().join(" ")));
 //	KSIMDEBUG(QString("Languages: %1").arg(KGlobal::locale()->languages()));
-//	KGlobal::locale()->setLanguage("C");
+//	KGlobal::locale()->setLanguage(QString::fromLatin1("C"));
 
 	if (!g_library)
 	{
@@ -153,7 +190,6 @@ KSimusApp::KSimusApp(QWidget* , const char* name)
 		loadLib = true;
 	}
 
-	config=kapp->config();
 
 	
 	
@@ -174,18 +210,24 @@ KSimusApp::KSimusApp(QWidget* , const char* name)
 		m_pastAllowed = true;
   }
 	slotSetupActions();
-//	fileSave->setEnabled(false);
-//	fileSaveAs->setEnabled(false);
-//	filePrint->setEnabled(false);
-//	editCut->setEnabled(false);
 
 	if(loadLib)
 	{
-		for (unsigned int i = 0; i < g_library->getMessages().count(); i++)
+		QStringList::ConstIterator it;
+		for (it = g_library->getInfoMessages().begin();
+		     it != g_library->getInfoMessages().end();
+		     ++it)
 		{
-    	LogListItem * item = new LogListItem(g_library->getMessages()[i], LOG_INFO);
-    	getLogList()->append(item);
+			getLogList()->info(*it);
 		}
+		for (it = g_library->getErrorMessages().begin();
+		     it != g_library->getErrorMessages().end();
+		     ++it)
+		{
+			getLogList()->error(*it);
+		}
+
+		g_library->clearMessageLists();
 	}
 }
 
@@ -198,7 +240,7 @@ KSimusApp::~KSimusApp()
 
 void KSimusApp::initActions()
 {
-	fileNewWindow = new KAction(i18n("New &Window"), 0, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
+	fileNewWindow = new KAction(i18n("New &Window"), QString::null, 0, this, SLOT(slotFileNewWindow()), actionCollection(),"file_new_window");
 	fileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection());
 	fileOpen = KStdAction::open(this, SLOT(slotFileOpen()), actionCollection());
 	fileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection());
@@ -220,7 +262,7 @@ void KSimusApp::initActions()
 	viewToolBar = KStdAction::showToolbar(this, SLOT(slotViewToolBar()), actionCollection());
 	viewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection());
 	viewLogView = new KToggleAction(i18n("&Log View"), CTRL+Key_B, this, SLOT(slotViewToggleLog()), actionCollection(), "view_toggle_log");
-	viewTreeView = new KToggleAction(i18n("&Component Supplier"), CTRL+Key_T, this, SLOT(slotViewToggleTree()), actionCollection(), "view_toggle_tree");
+	viewTreeView = new KToggleAction(i18n("Lis&t View"), CTRL+Key_T, this, SLOT(slotViewToggleList()), actionCollection(), "view_toggle_list");
 	viewTreeFold = new KAction(i18n("&Fold Supplier"), 0, this, SLOT(slotViewFoldTree()), actionCollection(), "view_tree_fold");
 	viewTreeUnfold = new KAction(i18n("&Unfold Supplier"), 0, this, SLOT(slotViewUnfoldTree()), actionCollection(), "view_tree_unfold");
 	
@@ -228,12 +270,16 @@ void KSimusApp::initActions()
 	settingMapSize = new KAction(i18n("Map &Sizes"), 0, this, SLOT(slotSettingMapSize()), actionCollection(), "setting_mapSize");
 	settingTiming = new KAction(i18n("&Timing"), 0, this, SLOT(slotSettingTiming()), actionCollection(), "setting_timing");
 	settingGrid = new KAction(i18n("&Grid"), 0, this, SLOT(slotSettingGrid()), actionCollection(), "setting_grid");
-  settingPackageFiles  = new KAction(i18n("&Package Files"), 0, this, SLOT(slotSettingPackageFiles()), actionCollection(), "setting_packageFiles");
+	settingPackageFiles  = new KAction(i18n("&Package Files"), 0, this, SLOT(slotSettingPackageFiles()), actionCollection(), "setting_packageFiles");
+	settingModuleFiles  = new KAction(i18n("&Module Files"), 0, this, SLOT(slotSettingModuleFiles()), actionCollection(), "setting_moduleFiles");
+	settingWatchWidget  = new KAction(i18n("&Watch"), 0, this, SLOT(slotSettingWatchWidget()), actionCollection(), "setting_watchWidget");
+	settingLogWidget  = new KAction(i18n("&Log View Properties"), 0, this, SLOT(slotSettingLogWidget()), actionCollection(), "setting_logWidget");
+	
 	
 	executeReset = new KAction(i18n("&Reset"), 0, this, SLOT(slotExecuteReset()), actionCollection(), "execute_reset");
-	executeStart = new KAction(i18n("&Start"), "exec", 0, this, SLOT(slotExecuteStart()), actionCollection(), "execute_start");
-	executeStop = new KAction(i18n("S&top"), "stop", 0, this, SLOT(slotExecuteStop()), actionCollection(), "execute_stop");
-	executePause = new KToggleAction(i18n("&Pause"), 0, this, SLOT(slotExecutePause()), actionCollection(), "execute_pause");
+	executeStart = new KAction(i18n("&Start"), QString::fromLatin1("exec"), 0, this, SLOT(slotExecuteStart()), actionCollection(), "execute_start");
+	executeStop = new KAction(i18n("S&top"), QString::fromLatin1("stop"), 0, this, SLOT(slotExecuteStop()), actionCollection(), "execute_stop");
+	executePause = new KToggleAction(i18n("&Pause"), QString::fromLatin1("player_pause"), 0, this, SLOT(slotExecutePause()), actionCollection(), "execute_pause");
 	
 //	testAction = new KAction(i18n("Test &Action"), 0, 0, this, SLOT(slotTestAction()), actionCollection(),"test_action");
 
@@ -268,12 +314,15 @@ void KSimusApp::initActions()
 	executeStart->setToolTip(i18n("Starts simulation"));
 	executeStop->setToolTip(i18n("Stops simulation"));
 	executePause->setToolTip(i18n("Pause simulation"));
-	executePause->setChecked(false);
+//	executePause->setChecked(false);
 
 	settingMapSize->setToolTip(i18n("Setup Schematic und User Interface size"));
 	settingTiming->setToolTip(i18n("Setup simulation timing"));
 	settingGrid->setToolTip(i18n("Setup grid"));
 	settingPackageFiles->setToolTip(i18n("Select packages"));
+	settingModuleFiles->setToolTip(i18n("Select modules"));
+	settingWatchWidget->setToolTip(i18n("General watch settings"));
+	settingLogWidget->setToolTip(i18n("Log Window Properties"));
 
 	connect(actionCollection(),SIGNAL(actionStatusText(const QString &)),SLOT(slotStatusHelpMsg(const QString &)));
 	actionCollection()->setHighlightingEnabled(true);
@@ -313,7 +362,7 @@ void KSimusApp::initView()
 	setCentralWidget(baseWin);
 	setCaption(getDocument()->URL().fileName(),false);
 
-	connect(baseWin->getTreeWidget(), SIGNAL(signalHideMe()), this, SLOT(slotViewHideTree()));
+	connect(baseWin->getComponentListWidget(), SIGNAL(signalHideMe()), this, SLOT(slotViewHideList()));
 
 	QBoxLayout * lay = new QHBoxLayout(baseWin->getWorkingWidget());
 	m_p->tabWidget = new QTabWidget(baseWin->getWorkingWidget());
@@ -364,17 +413,29 @@ void KSimusApp::initView()
 
 void KSimusApp::openDocumentFile(const KURL& url)
 {
-  slotStatusMsg(i18n("Opening file..."));
+	slotStatusMsg(i18n("Opening file..."));
 
-  doc->openDocument( url);
-  fileOpenRecent->addURL( url );
-  slotStatusMsg(i18n("Ready."));
+	doc->openDocument( url);
+	setCaption(url.fileName(), false);
+	fileOpenRecent->addURL( url );
+	slotStatusMsg(i18n("Ready."));
+}
+
+
+void KSimusApp::executeDocumentFile(const KURL& url)
+{
+	slotStatusMsg(i18n("Execute file..."));
+
+	doc->openDocument( url);
+	setCaption(url.fileName(), false);
+	fileOpenRecent->addURL( url );
+	slotExecuteStart();
 }
 
 
 KSimusDoc *KSimusApp::getDocument() const
 {
-   return doc;
+	return doc;
 }
 
 KSimEditor * KSimusApp::getActiveEditor() const
@@ -385,22 +446,35 @@ KSimEditor * KSimusApp::getActiveEditor() const
 /** returns a pointer to the log list widget */	
 LogList * KSimusApp::getLogList() const
 {
-  if(baseWin)
-	  return baseWin->getLogWidget();
-	return 0;
+	return (baseWin) ? baseWin->getLogWidget() : (LogList *)0;
+}
+
+/** returns a pointer to the watch widget */	
+WatchWidget * KSimusApp::getWatchWidget() const
+{
+	return (baseWin) ? baseWin->getWatchWidget() : (WatchWidget *)0;
 }
 
 void KSimusApp::saveOptions()
-{	
+{
 	config->setGroup("General Options");
 	config->writeEntry("First Start",false);
 	config->writeEntry("Geometry", size());
 	config->writeEntry("Show Toolbar", viewToolBar->isChecked());
 	config->writeEntry("Show Statusbar",viewStatusBar->isChecked());
 	config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
-	fileOpenRecent->saveEntries(config,"Recent Files");
+	fileOpenRecent->saveEntries(config,QString::fromLatin1("Recent Files"));
 	
 	config->writeEntry("Display Costmap",viewCostmap->isChecked());
+
+	// Watch options
+	config->setGroup("Watch Options");
+	config->writeEntry("Trace Size",getWatchWidget()->getTraceSize());
+	config->writeEntry("Rise by Add",getWatchWidget()->isRiseByAdd());
+	
+	// Log options
+	config->setGroup("Log Options");
+	getLogList()->save(*config);
 }
 
 
@@ -409,27 +483,6 @@ void KSimusApp::readOptions()
 	
 	config->setGroup("General Options");
 	
-	bool firstRead = config->readBoolEntry("First Start",true);
-	
-	if (firstRead)
-	{
-		// Some presets
-		QString group(config->group());
-	
-		// Default package dirs
-		config->setGroup("Packages");
-		KStandardDirs dirs;
-		QStringList packList = dirs.findDirs("lib","");
-		unsigned int i;
-		for (i=0; i<packList.count(); i++)
-		{
-			packList[i] += "ksimus/";
-		}
-		
-		config->writeEntry("Directories", packList);
-
-		config->setGroup(group);
-	}	
 
 	// bar status settings
 	bool bViewToolbar = config->readBoolEntry("Show Toolbar", true);
@@ -447,7 +500,7 @@ void KSimusApp::readOptions()
 	toolBar("mainToolBar")->setBarPos(toolBarPos);
 	
 	// initialize the recent file list
-	fileOpenRecent->loadEntries(config,"Recent Files");
+	fileOpenRecent->loadEntries(config,QString::fromLatin1("Recent Files"));
 
 	QSize size=config->readSizeEntry("Geometry");
 	if(!size.isEmpty())
@@ -456,61 +509,76 @@ void KSimusApp::readOptions()
 	}
 
 	viewCostmap->setChecked(config->readBoolEntry("Display Costmap",false));
-  doc->setCostMapVisible(viewCostmap->isChecked());
+	doc->setCostMapVisible(viewCostmap->isChecked());
+
+	
+	// Watch options
+	config->setGroup("Watch Options");
+	if (config->hasKey("Trace Size"))
+	{
+		getWatchWidget()->setTraceSize(config->readUnsignedNumEntry("Trace Size"));
+	}
+	if (config->hasKey("Rise by Add"))
+	{
+		getWatchWidget()->setRiseByAdd(config->readBoolEntry("Rise by Add"));
+	}
+
+	// Log options
+	config->setGroup("Log Options");
+	getLogList()->load(*config);
 }
 
 void KSimusApp::saveProperties(KConfig *_cfg)
 {
-  if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
-  {
-    // saving to tempfile not necessary
-
-  }
-  else
-  {
-    KURL url=doc->URL();	
-    _cfg->writeEntry("filename", url.url());
-    _cfg->writeEntry("modified", doc->isModified());
-    QString tempname = kapp->tempSaveName(url.url());
-    QString tempurl= KURL::encode_string(tempname);
-    KURL _url(tempurl);
-    doc->saveDocument(_url);
-  }
+	if(doc->URL().fileName()!=i18n("Untitled") && !doc->isModified())
+	{
+		// saving to tempfile not necessary
+	}
+	else
+	{
+		KURL url=doc->URL();	
+		_cfg->writeEntry("filename", url.url());
+		_cfg->writeEntry("modified", doc->isModified());
+		QString tempname = kapp->tempSaveName(url.url());
+		QString tempurl= KURL::encode_string(tempname);
+		KURL _url(tempurl);
+		doc->saveDocument(_url);
+	}
 }
 
 
 void KSimusApp::readProperties(KConfig* _cfg)
 {
-  QString filename = _cfg->readEntry("filename", "");
-  KURL url(filename);
-  bool modified = _cfg->readBoolEntry("modified", false);
-  if(modified)
-  {
-    bool canRecover;
-    QString tempname = kapp->checkRecoverFile(filename, canRecover);
-    KURL _url(tempname);
-  	
-    if(canRecover)
-    {
-      doc->openDocument(_url);
-      doc->setModified();
-      setCaption(_url.fileName(),true);
-      QFile::remove(tempname);
-    }
-  }
-  else
-  {
-    if(!filename.isEmpty())
-    {
-      doc->openDocument(url);
-      setCaption(url.fileName(),false);
-    }
-  }
-}		
+	QString filename = _cfg->readEntry("filename", QString::fromLatin1(""));
+	KURL url(filename);
+	bool modified = _cfg->readBoolEntry("modified", false);
+	if(modified)
+	{
+		bool canRecover;
+		QString tempname = kapp->checkRecoverFile(filename, canRecover);
+		KURL _url(tempname);
+		
+		if(canRecover)
+		{
+			doc->openDocument(_url);
+			doc->setModified();
+			setCaption(_url.fileName(),true);
+			QFile::remove(tempname);
+		}
+	}
+	else
+	{
+		if(!filename.isEmpty())
+		{
+			doc->openDocument(url);
+			setCaption(url.fileName(),false);
+		}
+	}
+}
 
 bool KSimusApp::queryClose()
 {
-  return doc->saveModified();
+	return doc->saveModified();
 }
 
 bool KSimusApp::queryExit()
@@ -525,80 +593,82 @@ bool KSimusApp::queryExit()
 
 void KSimusApp::slotFileNewWindow()
 {
-  slotStatusMsg(i18n("Opening a new application window..."));
+	slotStatusMsg(i18n("Opening a new application window..."));
 	
-  KSimusApp *new_window= new KSimusApp();
-  new_window->show();
+	KSimusApp *new_window= new KSimusApp();
+	new_window->show();
 
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileNew()
 {
 	getActiveEditor()->setEditorMode(EM_SELECT);
-  slotStatusMsg(i18n("Creating new document..."));
+	slotStatusMsg(i18n("Creating new document..."));
 
-  if(!doc->saveModified())
-  {
-     // here saving wasn't successful
+	if(!doc->saveModified())
+	{
+		// here saving wasn't successful
+	}
+	else
+	{
+		doc->newDocument();
+		setCaption(doc->URL().fileName(), false);
+		fileOpenRecent->setCurrentItem(-1);
+	}
 
-  }
-  else
-  {	
-    doc->newDocument();		
-    setCaption(doc->URL().fileName(), false);
-    fileOpenRecent->setCurrentItem(-1);
-  }
-
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileOpen()
 {
-  slotStatusMsg(i18n("Opening file..."));
+	slotStatusMsg(i18n("Opening file..."));
 	
-  if(!doc->saveModified())
-  {
-     // here saving wasn't successful
+	if(!doc->saveModified())
+	{
+		// here saving wasn't successful
+	}
+	else
+	{
+		KFileDialog * dia = new KFileDialog (QString::fromLatin1(":<documentPath>"),
+		                        i18n("*.sim|KSimus files (*.sim)\n*|All files"), this, 0, true);
 
-  }
-  else
-  {	
-		KFileDialog * dia = new KFileDialog (":<documentPath>",
-        i18n("*.sim|KSimus files (*.sim)\n*|All files"), this, 0, true);
+		dia->setCaption(i18n("Open File..."));
+		dia->exec();
 
-	  dia->setCaption(i18n("Open File..."));
-  	dia->exec();
+		KURL url(dia->selectedURL());
 
-	  KURL url(dia->selectedURL());
+		delete dia;
 
-  	delete dia;
-
-    if(!url.isEmpty())
-    {
-      doc->openDocument(url);
-      setCaption(url.fileName(), false);
-      fileOpenRecent->addURL( url );
-    }
-  }
-  slotStatusMsg(i18n("Ready."));
+		if(!url.isEmpty())
+		{
+			doc->openDocument(url);
+			fileOpenRecent->addURL( url );
+		}
+		else
+		{
+			fileOpenRecent->setCurrentItem(-1);
+		}
+		setCaption(doc->URL().fileName(), false);
+	}
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileOpenRecent(const KURL& url)
 {
-  slotStatusMsg(i18n("Opening file..."));
+	slotStatusMsg(i18n("Opening file..."));
 	
-  if(!doc->saveModified())
-  {
-     // here saving wasn't successful
-  }
-  else
-  {
-    doc->openDocument(url);
-    setCaption(url.fileName(), false);
-  }
+	if(!doc->saveModified())
+	{
+		// here saving wasn't successful
+	}
+	else
+	{
+		doc->openDocument(url);
+		setCaption(url.fileName(), false);
+	}
 
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileSave()
@@ -618,173 +688,180 @@ void KSimusApp::slotFileSave()
 
 void KSimusApp::slotFileSaveAs()
 {
-  slotStatusMsg(i18n("Saving file with a new filename..."));
+	slotStatusMsg(i18n("Saving file with a new filename..."));
 
-	KFileDialog * dia = new KFileDialog (":<documentPath>",
-        i18n("*.sim|KSimus files (*.sim)\n*|All files"), this, 0, true);
+	KFileDialog * dia = new KFileDialog (QString::fromLatin1(":<documentPath>"),
+	                        i18n("*.sim|KSimus files (*.sim)\n*|All files"), this, 0, true);
 
-  dia->setCaption(i18n("Save as..."));
-  dia->exec();
+	dia->setCaption(i18n("Save as..."));
+	dia->exec();
 
-  KURL url(dia->selectedURL());
-  QString filter(dia->currentFilter());
+	KURL url(dia->selectedURL());
+	QString filter(dia->currentFilter());
 
-  delete dia;
+	delete dia;
 
-  if(!url.isEmpty())
-  {
-    QString filename(url.fileName());
+	if(!url.isEmpty())
+	{
+		QString filename(url.fileName());
 
-    // Append extension, if no is present
-    if ((filename.right(4) != ".sim") && (filter == "*.sim"))
-    {
-    	filename += ".sim";
-    	url.setFileName(filename);
-    }
+		// Append extension, if no is present
+		if ((filename.right(4) != QString::fromLatin1(".sim"))
+		     && (filter == QString::fromLatin1("*.sim")))
+		{
+			filename += QString::fromLatin1(".sim");
+			url.setFileName(filename);
+		}
 
-    doc->saveDocument(url);
-    fileOpenRecent->addURL(url);
-    setCaption(url.fileName(),doc->isModified());
-  }
+		doc->saveDocument(url);
+		fileOpenRecent->addURL(url);
+		setCaption(url.fileName(),doc->isModified());
+	}
 
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileClose()
 {
-  slotStatusMsg(i18n("Closing file..."));
-	
-  close();
+	slotStatusMsg(i18n("Closing file..."));
+	doc->deleteContents();
+	close();
 
-  slotStatusMsg(i18n("Ready."));
+	//slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFilePrint()
 {
-  slotStatusMsg(i18n("Printing..."));
+	slotStatusMsg(i18n("Printing..."));
 
-  KPrinter printer;
-  if (printer.setup(this))
-  {
-    doc->getActiveView()->print(&printer);
-  }
+	KPrinter printer;
+	if (printer.setup(this))
+	{
+		doc->getActiveView()->print(&printer);
+	}
 
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotFileQuit()
 {
-  slotStatusMsg(i18n("Exiting..."));
-  saveOptions();
-  // close the first window, the list makes the next one the first again.
-  // This ensures that queryClose() is called on each window to ask for closing
-  KMainWindow* w;
-  if(memberList)
-  {
-    for(w=memberList->first(); w!=0; w=memberList->first())
-    {
-      // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
-      // the window and the application stay open.
-      if(!w->close())
-	break;
-    }
-  }	
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Exiting..."));
+	saveOptions();
+	// close the first window, the list makes the next one the first again.
+	// This ensures that queryClose() is called on each window to ask for closing
+	KMainWindow* w;
+	if(memberList)
+	{
+#if QT_VERSION >= 200 && QT_VERSION < 300
+// version 2.x
+		for(w=memberList->first(); w!=0; w=memberList->first())
+#else
+// version 3.x
+		for(w=memberList->first(); w!=0; w=memberList->next())
+#endif
+		{
+			// only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
+			// the window and the application stay open.
+			if(!w->close())
+				break;
+		}
+	}
+//	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotEditUndo()
 {
-  setFocus(); // For KSimLineEdit (Last undo)
-  getDocument()->undo();
-  if (m_p->currentView == APP_MODULE_VIEW)
-  {
+	setFocus(); // For KSimLineEdit (Last undo)
+	getDocument()->undo();
+	if (m_p->currentView == APP_MODULE_VIEW)
+	{
 		m_p->moduleDialog->setup();
-  }
+	}
 }
 
 void KSimusApp::slotEditRedo()
 {
-  getDocument()->redo();
-  if (m_p->currentView == APP_MODULE_VIEW)
-  {
+	getDocument()->redo();
+	if (m_p->currentView == APP_MODULE_VIEW)
+	{
 		m_p->moduleDialog->setup();
-  }
+	}
 }
 
 void KSimusApp::slotEditDelete()
 {
-  slotStatusHelpMsg(i18n("Deleting selection..."));
-  getActiveEditor()->deleteSelection();
+	slotStatusHelpMsg(i18n("Deleting selection..."));
+	getActiveEditor()->deleteSelection();
 }
 
 void KSimusApp::slotEditCut()
 {
-  slotStatusHelpMsg(i18n("Cutting selection..."));
-  getActiveEditor()->cutSelection();
-  editPaste->setEnabled(true);
-  m_pastAllowed = true;
+	slotStatusHelpMsg(i18n("Cutting selection..."));
+	getActiveEditor()->cutSelection();
+	editPaste->setEnabled(true);
+	m_pastAllowed = true;
 }
 
 void KSimusApp::slotEditCopy()
 {
-  slotStatusHelpMsg(i18n("Copying selection to clipboard..."));
-  getActiveEditor()->copySelection();
-  editPaste->setEnabled(true);
-  m_pastAllowed = true;
+	slotStatusHelpMsg(i18n("Copying selection to clipboard..."));
+	getActiveEditor()->copySelection();
+	editPaste->setEnabled(true);
+	m_pastAllowed = true;
 }
 
 void KSimusApp::slotEditPaste()
 {
-  slotStatusHelpMsg(i18n("Inserting clipboard contents..."));
-  getActiveEditor()->setEditorMode(EM_PAST);
+	slotStatusHelpMsg(i18n("Inserting clipboard contents..."));
+	getActiveEditor()->setEditorMode(EM_PAST);
 }
 
 void KSimusApp::slotEditInsertModule()
 {
-  slotStatusMsg(i18n("Inserting a module..."));
+	slotStatusMsg(i18n("Inserting a module..."));
 	QString file;
 
-	file=KFileDialog::getOpenFileName(":<modulePath>",
-                                    i18n("*.sim|KSimus files (*.sim)\n*|All files"),
-                                    this, i18n("Insert Module..."));
+	file=KFileDialog::getOpenFileName(QString::fromLatin1(":<modulePath>"),
+	                                  i18n("*.sim|KSimus files (*.sim)\n*|All files"),
+	                                  this, i18n("Insert Module..."));
 	if(!file.isEmpty())
 	{
 		QFileInfo fileInfo(file);
 		m_p->moduleFile = file;
-		slotSelectCI(&ModuleBaseInfo);
+		slotSelectCI(getModuleBaseInfo());
 	}
 
-  slotStatusMsg(i18n("Ready."));
+	slotStatusMsg(i18n("Ready."));
 }
 
 void KSimusApp::slotViewToolBar()
 {
-  slotStatusHelpMsg(i18n("Toggling toolbar..."));
-  ///////////////////////////////////////////////////////////////////
-  // turn Toolbar on or off
-  if(!viewToolBar->isChecked())
-  {
-    toolBar("mainToolBar")->hide();
-  }
-  else
-  {
-    toolBar("mainToolBar")->show();
-  }		
+	slotStatusHelpMsg(i18n("Toggling toolbar..."));
+	///////////////////////////////////////////////////////////////////
+	// turn Toolbar on or off
+	if(!viewToolBar->isChecked())
+	{
+		toolBar("mainToolBar")->hide();
+	}
+	else
+	{
+		toolBar("mainToolBar")->show();
+	}
 }
 
 void KSimusApp::slotViewStatusBar()
 {
-  slotStatusHelpMsg(i18n("Toggle the statusbar..."));
-  ///////////////////////////////////////////////////////////////////
-  //turn Statusbar on or off
-  if(!viewStatusBar->isChecked())
-  {
-    statusBar()->hide();
-  }
-  else
-  {
-    statusBar()->show();
-  }
+	slotStatusHelpMsg(i18n("Toggle the statusbar..."));
+	///////////////////////////////////////////////////////////////////
+	//turn Statusbar on or off
+	if(!viewStatusBar->isChecked())
+	{
+		statusBar()->hide();
+	}
+	else
+	{
+		statusBar()->show();
+	}
 }
 
 void KSimusApp::slotViewToggleLog()
@@ -814,33 +891,34 @@ void KSimusApp::slotViewHideLog()
 	}
 }
 
-void KSimusApp::slotViewToggleTree()
+void KSimusApp::slotViewToggleList()
 {
-	slotStatusHelpMsg(i18n("Toggle the component supplier..."));
 	///////////////////////////////////////////////////////////////////
-	//turn component supplier on or off
+	//turn list view on or off
 	if(!viewTreeView->isChecked())
 	{
-		baseWin->hideTreeWidget();
+//	slotStatusHelpMsg(i18n("Hide the list view..."));
+		baseWin->hideListWidget();
 		viewTreeFold->setEnabled(false);
 		viewTreeUnfold->setEnabled(false);
 	}
 	else
 	{
-		baseWin->showTreeWidget();
+//	slotStatusHelpMsg(i18n("Show the list view..."));
+		baseWin->showListWidget();
 		viewTreeFold->setEnabled(true);
 		viewTreeUnfold->setEnabled(true);
 	}
 }
 
-void KSimusApp::slotViewHideTree()
+void KSimusApp::slotViewHideList()
 {
-	slotStatusHelpMsg(i18n("Hide the component supplier..."));
+//	slotStatusHelpMsg(i18n("Hide the list view..."));
 	///////////////////////////////////////////////////////////////////
-	//turn component supplier off
+	//turn list view off
 	if(viewTreeView->isChecked())
 	{
-		baseWin->hideTreeWidget();
+		baseWin->hideListWidget();
 		viewTreeFold->setEnabled(false);
 		viewTreeUnfold->setEnabled(false);
 		viewTreeView->setChecked(false);
@@ -852,7 +930,7 @@ void KSimusApp::slotViewFoldTree()
 	slotStatusHelpMsg(i18n("Fold component supplier..."));
 	///////////////////////////////////////////////////////////////////
 	// fold component supplier
-	baseWin->getTreeWidget()->slotFoldTree();
+	baseWin->getComponentListWidget()->slotFoldTree();
 }
 
 void KSimusApp::slotViewUnfoldTree()
@@ -860,7 +938,7 @@ void KSimusApp::slotViewUnfoldTree()
 	slotStatusHelpMsg(i18n("Unfold component supplier..."));
 	///////////////////////////////////////////////////////////////////
 	// fold component supplier
-	baseWin->getTreeWidget()->slotUnfoldTree();
+	baseWin->getComponentListWidget()->slotUnfoldTree();
 }
 
 void KSimusApp::slotViewToggleCostmap()
@@ -903,21 +981,21 @@ void KSimusApp::slotSettingTiming()
 {
 	slotStatusMsg(i18n("Setup simulation timing..."));
 
-  KDialogBase * dia;
+	KDialogBase * dia;
 	dia = new KDialogBase(KDialogBase::TreeList,
-					i18n("Timings"),
-					KDialogBase::Default | KDialogBase::Ok | KDialogBase::Cancel,
- 					KDialogBase::Ok,
- 					this);
- 	QVBox * page;
+	                      i18n("Timing Settings"),
+	                      KDialogBase::Default | KDialogBase::Ok | KDialogBase::Cancel,
+	                      KDialogBase::Ok,
+	                      this);
+	QVBox * page;
 	SimulationTimingWidget * wid;
-	page = dia->addVBoxPage(i18n("Timing"));
+	page = dia->addVBoxPage(i18n("Timing Settings"));
 	wid = new SimulationTimingWidget(getDocument()->getTiming(), page, "Information");
 	connect(dia, SIGNAL(okClicked()), wid, SLOT(slotAccept()));
 	connect(dia, SIGNAL(defaultClicked()), wid, SLOT(slotDefault()));
 	connect(dia, SIGNAL(cancelClicked()), wid, SLOT(slotCancel()));
 //	connect(wid, SIGNAL(signalChangeData()), dia, SLOT(slotDataChanged()));
-  dia->exec();
+	dia->exec();
 	
 
 	delete dia;
@@ -957,7 +1035,7 @@ void KSimusApp::slotSettingPackageFiles()
 {
 	slotStatusMsg(i18n("Setup package files..."));
 
-  KSimPackageFileDialog * dia = new KSimPackageFileDialog();
+	KSimPackageFileDialog * dia = new KSimPackageFileDialog();
 	
 	// Load last size
 	QString group(config->group());
@@ -982,6 +1060,86 @@ void KSimusApp::slotSettingPackageFiles()
 	slotStatusMsg(i18n("Ready."));
 }
 
+void KSimusApp::slotSettingModuleFiles()
+{
+	slotStatusMsg(i18n("Setup module files..."));
+
+	KSimModuleFileDialog * dia = new KSimModuleFileDialog();
+
+	// Load last size
+	QString group(config->group());
+	config->setGroup("Modules/File Dialog");
+	QSize size=config->readSizeEntry("Geometry");
+	config->setGroup(group);
+	if(!size.isEmpty())
+	{
+		dia->resize(size);
+	}
+
+	// Execute dialog
+	dia->exec();
+
+	// Save size
+	config->setGroup("Modules/File Dialog");
+	config->writeEntry("Geometry", dia->size());
+	config->setGroup(group);
+
+	// Delete dialog
+	delete dia;
+	slotStatusMsg(i18n("Ready."));
+}
+
+void KSimusApp::slotSettingWatchWidget()
+{
+	slotStatusMsg(i18n("Setup watch..."));
+
+	KDialogBase * dia;
+	dia = new KDialogBase(KDialogBase::TreeList,
+	                      i18n("Watch Settings"),
+	                      KDialogBase::Default | KDialogBase::Ok | KDialogBase::Cancel,
+	                      KDialogBase::Ok,
+	                      this);
+	QVBox * page;
+	WatchWidgetPropertyWidget * wid;
+	page = dia->addVBoxPage(i18n("Watch Settings"));
+	wid = new WatchWidgetPropertyWidget(getWatchWidget(), page, "watch setting");
+	connect(dia, SIGNAL(okClicked()), wid, SLOT(slotAccept()));
+	connect(dia, SIGNAL(defaultClicked()), wid, SLOT(slotDefault()));
+	connect(dia, SIGNAL(cancelClicked()), wid, SLOT(slotCancel()));
+//	connect(wid, SIGNAL(signalChangeData()), dia, SLOT(slotDataChanged()));
+	dia->exec();
+
+
+	delete dia;
+	slotStatusMsg(i18n("Ready."));
+}
+
+
+void KSimusApp::slotSettingLogWidget()
+{
+	slotStatusMsg(i18n("Setup log window..."));
+
+	KDialogBase * dia;
+	dia = new KDialogBase(KDialogBase::TreeList,
+	                      i18n("Log Window Properties"),
+	                      KDialogBase::Default | KDialogBase::Ok | KDialogBase::Cancel,
+	                      KDialogBase::Ok,
+	                      this);
+	QVBox * page;
+	LogListDialogWidget * wid;
+	page = dia->addVBoxPage(i18n("Log Window Properties"));
+	wid = new LogListDialogWidget(getLogList(), page, "log property");
+	connect(dia, SIGNAL(okClicked()), wid, SLOT(slotAccept()));
+	connect(dia, SIGNAL(defaultClicked()), wid, SLOT(slotDefault()));
+	connect(dia, SIGNAL(cancelClicked()), wid, SLOT(slotCancel()));
+	dia->exec();
+
+
+	delete dia;
+	slotStatusMsg(i18n("Ready."));
+}
+
+
 void KSimusApp::slotExecuteReset()
 {
 	slotStatusHelpMsg(i18n("Resets simulation..."));
@@ -993,25 +1151,35 @@ void KSimusApp::slotExecuteStart()
 {
 	slotStatusHelpMsg(i18n("Starts simulation..."));
 
-	getDocument()->simulationStart();
-
-	baseWin->getTreeWidget()->setEnabled(false);
+	if (getDocument()->isSimulationPaused())
+	{
+		getDocument()->simulationPause(false);
+	}
+	else
+	{
+		getDocument()->simulationStart();
+		baseWin->getComponentListWidget()->setEnabled(false);
+	}
+	
 	slotSetupActions();
 }
 void KSimusApp::slotExecuteStop()
 {
 	slotStatusHelpMsg(i18n("Stops simulation..."));
-
-	executePause->setChecked(false);
+	
+//	executePause->setChecked(false);
 	
 	getDocument()->simulationStop();
 
 	slotSetupActions();
-	baseWin->getTreeWidget()->setEnabled(true);
+	baseWin->getComponentListWidget()->setEnabled(true);
 }
 void KSimusApp::slotExecutePause()
 {
 	slotStatusMsg(i18n("Pause simulation..."));
+	
+	getDocument()->simulationPause(true);
+	
 	slotSetupActions();
 	
 	slotStatusMsg(i18n("Ready."));
@@ -1020,18 +1188,31 @@ void KSimusApp::slotExecutePause()
 	
 void KSimusApp::slotStatusMsg(const QString &text)
 {
-  ///////////////////////////////////////////////////////////////////
-  // change status message permanently
-  statusBar()->clear();
-  statusBar()->changeItem(text, ID_STATUS_MSG);
+	///////////////////////////////////////////////////////////////////
+	// change status message permanently
+	statusBar()->clear();
+	statusBar()->changeItem(text, ID_STATUS_MSG);
+	m_p->messageTimer->stop();
+}
+
+void KSimusApp::slotStatusMsgDefault()
+{
+	///////////////////////////////////////////////////////////////////
+	// change status message to default
+	statusBar()->clear();
+	statusBar()->changeItem(i18n("Ready."), ID_STATUS_MSG);
+	m_p->messageTimer->stop();
 }
 
 void KSimusApp::slotStatusHelpMsg(const QString &text)
 {
-  ///////////////////////////////////////////////////////////////////
-  // change status message of whole statusbar temporary (text, msec)
-  statusBar()->message(text, 2000);
-//  KSIMDEBUG(text);
+	///////////////////////////////////////////////////////////////////
+	// change status message of whole statusbar temporary (text, msec)
+//	statusBar()->message(text, 2000);//  KSIMDEBUG(text);
+	statusBar()->clear();
+	statusBar()->changeItem(text, ID_STATUS_MSG);
+	m_p->messageTimer->start(2000, true);
+
 }
 
 
@@ -1150,7 +1331,7 @@ void KSimusApp::slotEditorModeChanged(int mode)
 	
 	if (((EditorModeType)mode != EM_INSERT) && ((EditorModeType)mode != EM_INSERT_MOVE))
 	{
-		baseWin->getTreeWidget()->slotUnselect();
+		baseWin->getComponentListWidget()->slotUnselect();
 	}
 }
 
@@ -1193,7 +1374,7 @@ void KSimusApp::slotViewChanged(QWidget * wid)
 void KSimusApp::slotSimulationTimeChanged()
 {
   QString s;
-  s = i18n("Time") + ": " + getDocument()->getTimeString();
+  s = i18n("Time") + QString::fromLatin1(": ") + getDocument()->getTimeString();
   statusBar()->changeItem(s, ID_TIME_LABEL);
 }
 
@@ -1205,7 +1386,7 @@ const QString & KSimusApp::getModuleFile() const
 
 const char * KSimusApp::convertViewType(eAppViewType view)
 {
-	return currentViewDict.find(view);
+	return getCurrentViewDict().find(view);
 }
 
 
@@ -1236,7 +1417,7 @@ void KSimusApp::setCurrentView(eAppViewType newView)
 
 void KSimusApp::setCurrentView(const char * newView)
 {
-	setCurrentView(currentViewDict.find(newView,APP_SHEET_VIEW));
+	setCurrentView(getCurrentViewDict().find(newView,APP_SHEET_VIEW));
 }
 
 eAppViewType KSimusApp::getCurrentView() const
@@ -1252,6 +1433,7 @@ const char * KSimusApp::getCurrentViewString() const
 void KSimusApp::slotSetupActions()
 {
 	bool running = getDocument()->isSimulationRunning();
+	bool paused  = getDocument()->isSimulationPaused();
 	bool stopped = !running;
 	bool editView = (getCurrentView() != APP_MODULE_VIEW);
 	
@@ -1278,13 +1460,20 @@ void KSimusApp::slotSetupActions()
 //	viewLogView->setEnabled(ena);		always enabled
 //	viewTreeView->setEnabled(ena);		always enabled
 	viewCostmap->setEnabled(stopped);
+	viewTreeFold->setEnabled(stopped);
+	viewTreeUnfold->setEnabled(stopped);
+	
 	settingTiming->setEnabled(stopped);
+	settingMapSize->setEnabled(stopped);
+	settingPackageFiles->setEnabled(stopped);
+	settingModuleFiles->setEnabled(stopped);
 	settingGrid->setEnabled(stopped && editView);
 	
-//	executeReset->setEnabled(ena);		always enabled
-	executeStart->setEnabled(stopped);
+	executeReset->setEnabled(stopped);
+	executeStart->setEnabled(stopped || paused);
 	executeStop->setEnabled(running);
-	executePause->setEnabled(false);
+	executePause->setEnabled(running && !paused);
+	executePause->setChecked(paused);
 
 //	testAction->setEnabled(true);		always enabled
 }
