@@ -20,9 +20,14 @@
 #include <qrect.h>
 #include <qsize.h>
 #include <qpopupmenu.h>
+#include <qobjectlist.h>
+
 #include <kfiledialog.h>
 #include <klocale.h>
 
+#include "ksimusdoc.h"
+#include "ksimusview.h"
+#include "ksimeditor.h"
 #include "ksimdata.h"
 #include "ksimdebug.h"
 #include "module.h"
@@ -36,9 +41,10 @@
 #include "displaywidget.h"
 #include "ksimwidget.h"
 #include "externalconnector.h"
+#include "optionalconnector.h"
 
 static const char * sModFile = "ModuleFile";
-//static const char * sConnector = "Connector";
+
 #define minX	8
 #define minY	7
 
@@ -50,11 +56,16 @@ static Component * create(CompContainer * container, const ComponentInfo * ci)
 	return new Module(container, ci);
 }
 
-const ComponentInfo ModuleBaseInfo (	"Module",
-									"Module",
-									0,
-									VA_SHEET_AND_USER,
-									create );
+const ComponentInfo * getModuleBaseInfo()
+{
+	static const ComponentInfo Info(QString::fromLatin1("Module"),
+	                                QString::fromLatin1("Module"),
+	                                i18n("Component", "Module"),
+	                                QString::null,
+	                                VA_SHEET_AND_USER,
+	                                create);
+	return &Info;
+}
 
 //###############################################################
 
@@ -63,27 +74,80 @@ ModuleSV::ModuleSV(Component * comp, eViewType viewType)
 {
 	widgetList = new KSimWidgetList();
 	CHECK_PTR(widgetList);
-	setPlace(QRect(0, 0, minX*gridX, minY*gridY));
+
+	if(viewType == SHEET_VIEW)
+	{
+		setPlace(QRect(0, 0, minX * gridX, minY * gridY));
+		enableConnectorSpacingRight(true);
+		enableConnectorSpacingTop(true);
+		enableConnectorSpacingBottom(true);
+		enableConnectorSpacingLeft(true);
+	}
+	else
+	{
+		setPlace(QRect(0, 0, (minX-2) * gridX, (minY-2) * gridY));
+		enableConnectorSpacingRight(false);
+		enableConnectorSpacingTop(false);
+		enableConnectorSpacingBottom(false);
+		enableConnectorSpacingLeft(false);
+	}
 }
 ModuleSV::~ModuleSV()
 {
-	if (widgetList)
-		delete widgetList;
+	delete widgetList;
+}
+
+
+bool ModuleSV::load(KSimData & file)
+{
+	bool res = CompView::load(file);
+
+	switch(((Module *) getComponent())->getModuleContainer()->getModuleData()->getModuleView())
+	{
+		case MV_GENERIC:
+		case MV_PIXMAP:
+			if ((getViewType() == USER_VIEW) && !isHidden())
+			{
+				// always hidden !!!
+				setHide(true);
+			}
+			break;
+
+		case MV_USERVIEW:
+			// Nothing to do
+			break;
+
+		default:
+			KSIMDEBUG_VAR("Unknown module view",(int)((Module *) getComponent())->getModuleContainer()->getModuleData()->getModuleView());
+			break;
+	}
+
+	return res;
 }
 
 void ModuleSV::draw(QPainter * p)
 {
+	CompView::draw(p);
+	
 	Module * mod = (Module *) getComponent();
 	QSize size(getPlace().size());
 		
 	switch(mod->getModuleContainer()->getModuleData()->getModuleView())
 	{
 		case MV_GENERIC:
-			drawGeneric(p, size);
+			// Don't draw if sub module
+//			if  (mod->getContainer()->isParentDoc())
+			if (getViewType() == SHEET_VIEW)
+			{
+				drawGeneric(p, size);
+			}
 			break;
 			
 		case MV_PIXMAP:
-			p->drawPixmap(gridX, gridY, *mod->getModuleContainer()->getModuleData()->getPixmap());
+			if (getViewType() == SHEET_VIEW)
+			{
+				p->drawPixmap(gridX, gridY, *mod->getModuleContainer()->getModuleData()->getPixmap());
+			}
 			break;
 			
 		case MV_USERVIEW:
@@ -94,8 +158,6 @@ void ModuleSV::draw(QPainter * p)
 			KSIMDEBUG_VAR("Unknown module view",(int)mod->getModuleContainer()->getModuleData()->getModuleView());
 			break;
 	}
-	
-	CompView::draw(p);
 }
 
 void ModuleSV::drawGeneric(QPainter * p, QRect & place)
@@ -105,9 +167,9 @@ void ModuleSV::drawGeneric(QPainter * p, QRect & place)
 	p->setBrush(NoBrush);
 	p->drawRect(place.x() + gridX+1, place.y() + gridY+1, place.width()-gridX*2-1, place.height()-gridY*2-1);
 	p->drawText(place.x() + gridX+1, place.y() + gridY+1, place.width()-gridX*2-1, place.height()-gridY*2-1,
-	            AlignCenter, "Module");
+	            AlignCenter, QString::fromLatin1("Module"));
 	p->restore();
-}	
+}
 
 void ModuleSV::drawGeneric(QPainter * p, QSize & size)
 {
@@ -119,7 +181,7 @@ void ModuleSV::print(QPainter * paint)
 {
 	draw(paint);
 
-  QWidget * pWid;
+	QWidget * pWid;
 	if (widgetList && (pWid = widgetList->getFirstWidget()))
 	{
 		// Reset the compView translation (a little bit ugly :)
@@ -136,19 +198,11 @@ QWidget * ModuleSV::createCompViewWidget(QWidget * parent)
 	
 	Module * module = (Module*)getComponent();
 	
-	if (module->getModuleContainer()->getModuleData()->getModuleView() != MV_USERVIEW)
-	{
-		return (QWidget *)0;
-	}
-	
+//	KSIMDEBUG_VAR("ModuleSV::createCompViewWidget", getComponent()->getName());
 	CompViewList * viewList = module->getModuleContainer()->getUserViewList();
-	ModuleWidget * display = new ModuleWidget(module, viewList , parent, module->getName());
+	ModuleWidget * display = new ModuleWidget(module, viewList, parent, module->getName());
 	CHECK_PTR(display);
 	widgetList->addWidget(display);
-	display->setBackgroundMode(QWidget::PaletteDark);
-
-	display->installEventFilter(parent);
-	display->setMouseTracking(true);
 	
 	/* General signals */
 	// Delete signal (CompView->Widget)
@@ -156,449 +210,86 @@ QWidget * ModuleSV::createCompViewWidget(QWidget * parent)
 	// Delete signal (Editor->Widget)
 	QObject::connect(parent, SIGNAL(destroyed()), display, SLOT(slotDelete()));
 	// Move signal (CompView->Widget)
-	QObject::connect(this, SIGNAL(signalMove(const QPoint &)), display, SLOT(move(const QPoint &)));
+	QObject::connect(this, SIGNAL(signalMoveWidget(const QPoint &)), display, SLOT(move(const QPoint &)));
 	// Resize signal (CompView->Widget)
-	QObject::connect(this, SIGNAL(signalResize(const QSize &)), display, SLOT(resize(const QSize &)));
+	QObject::connect(this, SIGNAL(signalResizeWidget(const QSize &)), display, SLOT(resize(const QSize &)));
+
+	// Hide signal (CompView->ModuleSV)
+	connect(this, SIGNAL(signalHide()), this, SLOT(slotWidgetHide()));
+	// Hide signal (ModuleSV->Widget)
+	connect(this, SIGNAL(signalWidgetHide()), display, SLOT(hide()));
+
+	// Show signal (CompView->ModuleSV)
+	connect(this, SIGNAL(signalShow()), this, SLOT(slotWidgetShow()));
+	// Show signal (ModuleSV->Widget)
+	connect(this, SIGNAL(signalWidgetShow()), display, SLOT(show()));
 
-	// Hide signal (CompView->Widget)
-	connect(this, SIGNAL(signalHide()), display, SLOT(hide()));
-	// Show signal (CompView->Widget)
-	connect(this, SIGNAL(signalShow()), display, SLOT(show()));
-	
 	display->reload();
 	
-	//Work around for widget positioning
-	resize();
-	display->show();
-	
 	return display;
 }
 
-void ModuleSV::setPos(const QPoint & pos)
+void ModuleSV::drawBound(QPainter * p)
 {
-	CompView::setPos(pos);
-	
-	QPoint gridPos = mapToGrid(pos);
-	QPoint newPos (gridPos.x()+gridX, gridPos.y()+gridY);
-	emit signalMove(newPos);
-}
-
-void ModuleSV::resize()
-{
-	CompView::resize();	
-	
-	QSize newSize;
-	
-	newSize.setWidth(getPlace().width() - 2*gridX );
-	newSize.setHeight(getPlace().height() - 2*gridY );
-
-	emit signalResize(newSize);
-	setPos(getPlace().topLeft());
-	
+	// Draw boundary
+	if (((Module*)getComponent())->getModuleContainer()->getModuleData()->getModuleView() == MV_USERVIEW)
+	{
+		p->save();
+		p->setRasterOp (NotROP);
+		p->setPen (DashLine);
+		p->setBrush (NoBrush);
+		p->drawRect (getPlace().left()-1,
+	               getPlace().top()-1,
+	               getPlace().width()+2,
+	               getPlace().height()+2);
+		p->restore();
+	}
+	else
+	{
+		CompView::drawBound(p);
+	}
 }
 
 void ModuleSV::reload()
 {
 	QList<QWidget> * list = widgetList->getWidgetList();
+//	KSIMDEBUG_VAR("ModuleSV::reload()", list->count());
+
 	for(QListIterator<QWidget> it(*list);it.current();++it)
 	{
 		((ModuleWidget*)it.current())->reload();
 	}
 }
 
-//###############################################################
 
-
-
-Module::Module(CompContainer * _container, const ComponentInfo * ci)
-	: Component(_container, ci),
-		moduleFile()
+void ModuleSV::slotWidgetShow()
 {
-	m_isModule = true;
-	
-	m_moduleContainer = new CompContainer(this);
-	CHECK_PTR(m_moduleContainer);
-	
-	new ModuleSV(this, SHEET_VIEW);
-	new ModuleSV(this, USER_VIEW);
-	
-	extList = new ComponentList;
-	CHECK_PTR(extList);
-}
-Module::~Module()
-{
-//	delete container;
-	delete extList;
-}
-
-void Module::setModuleFile(const QString & fileName)
-{
-	moduleFile = fileName;
-	reloadModule();
-}
-	
-const QString & Module::getModuleFile() const
-{
-	return moduleFile;
-}
-	
-void Module::save(KSimData & file) const
-{
-	file.writeEntry(sModFile, moduleFile);
-//	file.writeEntry(sConnector, getConnList()->count());
-	
-	Component::save(file);
-}
-
-/** load module properties
-*   copyLoad is true, if the load function is used as a copy function
-*	Returns true if successful */
-bool Module::load(KSimData & file, bool copyLoad)
-{
-//	unsigned int connNo;//,i;
-	
-	moduleFile = file.readEntry(sModFile);
-//	connNo = file.readUnsignedNumEntry(sConnector,0);
-	
-	reloadModule();
-	
-	return Component::load(file, copyLoad);
-
-}
-	
-/** reloads the module and recreates the view */
-void Module::reloadModule()
-{
-	PointList * posList = 0;
-	unsigned int i;
-	ModuleData * mdata;
-	ConnectorList * oldConnList = new ConnectorList(*getConnList());
-	oldConnList->setAutoDelete(false);
-	
-	// Remove all information from container
-	m_moduleContainer->deleteAll();
-	mdata = m_moduleContainer->getModuleData();
-	
-	// First remove old connectors
-	getConnList()->setAutoDelete(false);	//Do not delete connectors
-	getConnList()->clear();								//Clear list
-	getConnList()->setAutoDelete(true);
-	extList->clear();
-	
-	
-	// open file
-	KSimData file(moduleFile);
-
-	// Load all components from file
-	file.setGroup("/");
-	getModuleContainer()->loadProperty(file);
-	getModuleContainer()->loadComponents(file);
-
-	setName(mdata->getModuleName());
-	
-	if ((mdata->getModuleView() == MV_PIXMAP) && (!mdata->isPixmapFileValid()))
-	{
-		logWarning(i18n("Pixmap file '%1' is not valid! Switch to generic view.").arg(mdata->getPixmapFile()));
-		mdata->setModuleView(MV_GENERIC);
-	}
-
-	if(getSheetView())
-		((ModuleSV*)getSheetView())->reload();
-	if(getUserView())
-		((ModuleSV*)getUserView())->reload();
-	
-	
-	// Search external connectors
-	mdata->searchExternals(m_moduleContainer->getComponentList());
-	
-	switch(mdata->getModuleView())
+	switch(((Module *) getComponent())->getModuleContainer()->getModuleData()->getModuleView())
 	{
 		case MV_GENERIC:
-		{
-		    // Calculate and set module size
-			mdata->setupGenericData();
-	
-			// set view size
-			if (getSheetView())
-			{
-				getSheetView()->setHide(false);
-				emit getSheetView()->signalHide();
-				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getGenericSize()));
-			}
-			
-			if (getUserView())
-			{
-				getUserView()->setHide(true);
-				emit getUserView()->signalHide();
-				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getGenericSize()));
-			}
-	
-			// Create Connectors
-			posList = mdata->getGenericConnPos();
-		}
-		break;
-		
 		case MV_PIXMAP:
-		{
-		    // Calculate and set module size
-			mdata->setupPixmapData();
-			mdata->loadPixmap();
-	
-			// set view size
-			if (getSheetView())
-			{
-				getSheetView()->setHide(false);
-				emit getSheetView()->signalHide();
-				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getPixmapSize()));
-			}
-			if (getUserView())
-			{
-				getUserView()->setHide(true);
-				emit getUserView()->signalHide();
-				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getPixmapSize()));
-			}
-	
-			// Create Connectors
-			posList = mdata->getPixmapConnPos();
-		}
-		break;
-		
+			// always hidden !!!
+			emit signalWidgetHide();
+			break;
+
 		case MV_USERVIEW:
-		{
-		    // Calculate and set module size
-			mdata->setupUserViewData();
-	
-			// set view size
-			if (getSheetView())
-			{
-				getSheetView()->setHide(false);
-				emit getSheetView()->signalShow();
-				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getUserViewSize()+QSize(2*gridX,2*gridY)));
-			}
-			if (getUserView())
-			{
-				if(mdata->getUserViewAttrib() == VA_SHEET_AND_USER)
-				{
-					getUserView()->setHide(false);
-					emit getUserView()->signalShow();
-				}
-				else
-				{
-					getUserView()->setHide(true);
-					emit getUserView()->signalHide();
-				}
-					
-				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getUserViewSize()+QSize(2*gridX,2*gridY)));
-			}
-			// Create Connectors
-			posList = mdata->getUserViewConnPos();
-		}
-		break;
-		
+			emit signalWidgetShow();
+			// Nothing to do
+			break;
+
 		default:
-			KSIMDEBUG_VAR("Unknown module view",(int)mdata->getModuleView());
+			KSIMDEBUG_VAR("Unknown module view",(int)((Module *) getComponent())->getModuleContainer()->getModuleData()->getModuleView());
 			break;
 	}
-
-	CHECK_PTR(posList);
-	for	(i=0; i < posList->count(); i++)
-	{
-		ExternalConnector * extConn = (ExternalConnector *)mdata->getExternalList()->at(i);
-		ConnectorBase *conn = searchConn (extConn, oldConnList);
-		
-		if (posList->at(i)->x() != -1)
-		{
-			if (conn)
-			{
-				// The connector exist
-				oldConnList->removeRef(conn);
-//				getConnList()->append(conn);
-				addConnector(conn);
-			}
-			else
-			{
-				// The connector doesn't exist. Create it
-/*				conn = extConn->getExternalConn()->getConnInfo()->create(this, "", QPoint(0,0));
-				conn->setName(extConn->getName());
-				conn->setNegate(extConn->getExternalConn()->isNegated(),true);*/
-				conn = extConn->getExternalConn()->getConnInfo()->create(this, extConn->getName(), QPoint(0,0));
-				conn->setWireName(QString("(extConn) %1").arg(extConn->getSerialNumber()));
-				conn->setNegate(extConn->getExternalConn()->isNegated(),true);
-			}
 	
-			extList->append(extConn);
-	
-			conn->setGridPos(*(posList->at(i)));
-	    	
-			if (mdata->getModuleView() == MV_PIXMAP)
-				conn->setOrientation(*mdata->getPixmapConnOrient()->at(i));
-		}
-		else
-		{
-			// Remove connection
-			if (conn && conn->getWire())
-			{
-				getContainer()->delConnection(conn);
-				oldConnList->removeRef(conn);
-				delete conn;
-			}
-		}
-	    	
-	}
-	
-	// Remove unused connecetors
-	oldConnList->setAutoDelete(true);
-	while(oldConnList->count())
-	{
-		if (oldConnList->first()->getWire())
-			getContainer()->delConnection(oldConnList->first());
-		oldConnList->removeFirst();
-	}
-	delete oldConnList;
 }
 
-/** search the connector
-	returns 0,if no connector is found */
-ConnectorBase * Module::searchConn(ExternalConnector * extConn, ConnectorList * connList)
+void ModuleSV::slotWidgetHide()
 {
-	const ConnectorInfo * extConnInfo = extConn->getExternalConn()->getConnInfo();
-//	QString extName = extConn->getName();
-	QString connWireName = QString("(extConn) %1").arg(extConn->getSerialNumber());
-	FOR_EACH_CONNECTOR(it, *connList)
-	{
-		// Compare connector type and name
-			if ((it.current()->getConnInfo() == extConnInfo)
-//			  &&(extName == it.current()->getName()))
-			  &&(connWireName == it.current()->getWireName()))
-		{
-			// Both have same type and connector name have the correct wire name
-			return it.current();
-		}
-	}
-	return (ConnectorBase*)0;
+	emit signalWidgetHide();
 }
 
 
-/** Checks the component
-*   eg. all required inputs are connected
-*   The implementation calls the checks functions of the connectors and container.
-*	Returns the number of errors
-*/
-int Module::checkCircuit()
-{
-	int errors = Component::checkCircuit();
-	
-	errors += m_moduleContainer->checkCircuit();
-	
-	return errors;
-}
-
-/** Executes the simulation of this component */
-void Module::calculate()
-{
-	Component::calculate();
-	
-	// First get all input signals
-	for(unsigned int i = 0; i < extList->count(); i++)
-	{
-   		ExternalConnector * extConn = (ExternalConnector*) extList->at(i);
-		// Input ?
-		if (extConn->isInput())
-		{
-    		ConnectorInputBase * inExternal = (ConnectorInputBase*) getConnList()->at(i);
-    		ConnectorOutputBase * outInternal = (ConnectorOutputBase*) extConn->getInternalConn();
-    		
-    		// Set data to wire
-    		const void * pData;
-    		pData = inExternal->readoutData();
-    		outInternal->copyData(pData);
-    	}
-    }
-    // Calculate components
-    FOR_EACH_COMP(it, *getModuleContainer()->getCalculateComponentList())
-    {
-    	it.current()->calculate();
-		
-			if (it.current()->getSheetView())
-			{
-				if (it.current()->getSheetView()->isViewChanged())
-				{
-					getSheetView()->setViewChanged(true);
-				}
-			}
-    }
-}
-
-/** Shift the result of calculation to output */
-void Module::updateOutput()
-{
-	Component::updateOutput();
-
-	// update outputs components
-	FOR_EACH_COMP(it, *m_moduleContainer->getUpdateOutputComponentList())
-	{
-		if (!it.current()->isExtConn())
-		{
-			it.current()->updateOutput();
-		}
-	}
-
-	// At last set all output signals
-	for(unsigned int i = 0; i < extList->count(); i++)
-	{
-   		ExternalConnector * extConn = (ExternalConnector*) extList->at(i);
-		// Output ?
-		if (!extConn->isInput())
-		{
-    		ConnectorInputBase * inInternal = (ConnectorInputBase*)extConn->getInternalConn();
-    		ConnectorOutputBase * outExternal = (ConnectorOutputBase*) getConnList()->at(i);
-    		
-    		// Set data to wire
-    		const void * pData;
-    		pData = inInternal->readoutData();
-    		outExternal->copyData(pData);
-    	}
-    }
-}
-/** Reset all simulation variables */
-void Module::reset()
-{
-	Component::reset();
-	// Call reset for all components
-	FOR_EACH_COMP(it,*m_moduleContainer->getComponentList())
-	{
-		it.current()->reset();
-	}
-}
-
-	
-/** Initialize the component popup menu */
-bool Module::initPopupMenu(QPopupMenu * popup)
-{
-	bool insert = Component::initPopupMenu(popup);
-	if (!insert)
-		popup->insertSeparator();
-	popup->insertItem((const char*)i18n("&Reload module"), this, SLOT(slotReload()));
-	return true;
-}
-
-/** Executes the reload function */
-void Module::slotReload()
-{
-	reloadModule();
-	refresh();
-//	getContainer()->routeComponents();
-//	getDoc()->slotUpdateAllViews(0);
-}
-
-/** Setup the component lists for calculation, updateOutput, updateView.
- 	* Call during simulation start. */
-void Module::setupSimulationList()
-{
-	getModuleContainer()->setupSimulationList();
-}
-
-	
 //###############################################################
 
 ModuleWidget::ModuleWidget(Module * module, CompViewList * viewList, QWidget *parent, const char *name)
@@ -627,24 +318,32 @@ bool ModuleWidget::eventFilter( QObject * obj, QEvent * ev)
 			case QEvent::MouseButtonDblClick:
 			case QEvent::MouseMove:
 			{
-				QMouseEvent mouseEv = QMouseEvent(  ev->type(),
-								 	 				((QWidget*)obj)->mapToParent(((QMouseEvent*)ev)->pos()),
-								  					((QMouseEvent*)ev)->button(),
-								  					((QMouseEvent*)ev)->state()
-								  				 );
+				QMouseEvent mouseEv = QMouseEvent(ev->type(),
+				                  ((QWidget*)obj)->mapToParent(((QMouseEvent*)ev)->pos()),
+				                  ((QMouseEvent*)ev)->button(),
+				                  ((QMouseEvent*)ev)->state()
+				                  );
 				return event(&mouseEv);
 			}
-			
+
 			default:
 				return false;
 		}
 	}
 }
-	
+
 void ModuleWidget::reload()
 {
+/*	KSIMDEBUG_VAR("ModuleWidget::reload", m_module->getName());
+
+	KSIMDEBUG_VAR("ModuleWidget::reload()", displayList->count());
+	KSIMDEBUG_VAR("ModuleWidget::reload()", m_module->getContainer()->getUserViewList()->count());
+	KSIMDEBUG_VAR("ModuleWidget::reload()", m_module->getModuleContainer()->getUserViewList()->count());*/
+
+	// Delete all child objects
 	emit QObject::destroyed();
-	
+
+	// Recreate child widgets if user view
 	if (m_module->getModuleContainer()->getModuleData()->getModuleView() == MV_USERVIEW)
 	{
 		FOR_EACH_COMPVIEW(it, *displayList)
@@ -652,5 +351,501 @@ void ModuleWidget::reload()
 			it.current()->makeWidget(this);
 		}
 	}
+
+/*	if (children())
+	{
+		QObjectListIt it(*children());
+		for (; it.current(); ++it)
+		{
+			if (it.current()->isWidgetType())
+			{
+				KSIMDEBUG(QString::fromLatin1("ModuleWidget::reload Module %1 Widget %2").arg(m_module->getName()).arg(QString::fromLatin1(it.current()->name())));
+			}
+		}
+	}*/
+
+/*	// Show display list
+	if (displayList)
+	{
+		FOR_EACH_COMPVIEW(it, *displayList)
+		{
+			KSIMDEBUG(QString::fromLatin1("ModuleWidget::reload displayList Module %1 Widget %2").arg(m_module->getName()).arg(QString::fromLatin1(it.current()->name())));
+		}
+	}*/
+
+}
+
+//###############################################################
+
+
+
+Module::Module(CompContainer * _container, const ComponentInfo * ci)
+	: Component(_container, ci),
+		moduleFile()
+{
+	setComponentType(eModule);
+	
+	m_moduleContainer = new CompContainer(this);
+	CHECK_PTR(m_moduleContainer);
+	
+	new ModuleSV(this, SHEET_VIEW);
+	new ModuleSV(this, USER_VIEW);
+	
+	extList = new ComponentList;
+	CHECK_PTR(extList);
+}
+Module::~Module()
+{
+	delete extList;
+}
+
+void Module::setModuleFile(const QString & fileName)
+{
+	moduleFile = fileName;
+	reloadModule();
+}
+	
+const QString & Module::getModuleFile() const
+{
+	return moduleFile;
+}
+	
+void Module::save(KSimData & file) const
+{
+	file.writeEntry(sModFile, moduleFile);
+	
+	Component::save(file);
+}
+
+/** load module properties
+*   copyLoad is true, if the load function is used as a copy function
+*	Returns true if successful */
+bool Module::load(KSimData & file, bool copyLoad)
+{
+	moduleFile = file.readEntry(sModFile);
+	
+	reloadModule();
+	
+	return Component::load(file, copyLoad);
+}
+	
+/** reloads the module and recreates the view */
+void Module::reloadModule()
+{
+	PointList * posList = (PointList *)0;
+	unsigned int i;
+	ModuleData * mdata;
+
+	// remove optional connector if exist
+	if (getAddOnList())
+	{
+		QListIterator<ComponentAddOn> it(*getAddOnList());
+		while(it.current())
+		{
+			if(it.current()->inherits("OptionalConnector"))
+			{
+				delete it.current();
+			}
+			else
+			{
+				++it;
+			}
+		}
+	}
+
+	ConnectorList * oldConnList = new ConnectorList(*getConnList());
+	oldConnList->setAutoDelete(false);
+
+	// Remove all information from container
+	m_moduleContainer->deleteAll();
+	mdata = m_moduleContainer->getModuleData();
+	
+	// First remove old connectors
+	getConnList()->setAutoDelete(false);  //Do not delete connectors
+	getConnList()->clear();               //Clear list
+	getConnList()->setAutoDelete(true);
+	extList->clear();
+	
+	QFileInfo fileInfo(getModuleFile());
+	
+	if (!fileInfo.exists())
+	{
+		logError(i18n("Module file %1 does not exist.").arg(getModuleFile()));
+	}
+	else if (!fileInfo.isReadable())
+	{
+		logError(i18n("Module file %1 is not readable.").arg(getModuleFile()));
+	}
+	else if (fileInfo.isDir())
+	{
+		logError(i18n("Module file %1 seems to be a directory.").arg(getModuleFile()));
+	}
+	else if (checkRecursion())
+	{
+		logError(i18n("Recursion detected in module file %1.\nRecursive modules are not allowed.").arg(getModuleFile()));
+	}
+	else
+	{
+		// open file read only
+		KSimData file(getModuleFile(), KSimData::versionAsIs, true);
+
+		// Load all components from file
+		file.setGroup("/");
+		getModuleContainer()->loadProperty(file);
+		getModuleContainer()->loadComponents(file);
+
+		setName(mdata->getModuleName());
+	}
+	
+	// Search external connectors
+	mdata->searchExternals(m_moduleContainer->getComponentList());
+
+	// Some view type depending checks and setup 
+	switch(mdata->getModuleView())
+	{
+		case MV_GENERIC:
+			// Nothing yet
+			break;
+
+		case MV_PIXMAP:
+			if (mdata->isPixmapFileValid())
+			{
+				// Calculate and set module size
+				mdata->setupPixmapData();
+				mdata->loadPixmap();
+				if (!mdata->checkConnectorPosition(true))
+				{
+					mdata->setModuleView(MV_GENERIC);
+				}
+			}
+			else
+			{
+				logError(i18n("Pixmap file '%1' is not valid! Switch to generic view.").arg(mdata->getPixmapFile()));
+				mdata->setModuleView(MV_GENERIC);
+			}
+			break;
+
+		case MV_USERVIEW:
+			// Calculate and set module size
+			mdata->setupUserViewData();
+			if (!mdata->checkConnectorPosition(true))
+			{
+				mdata->setModuleView(MV_GENERIC);
+			}
+			break;
+		
+		case MV_NONE:
+			logError(i18n("File %1 is not a module! Switch to generic view.").arg(getModuleFile()));
+			mdata->setModuleView(MV_GENERIC);
+			break;
+
+		default:
+			KSIMDEBUG(QString::fromLatin1("Unknown module view = %1 Filename = %2").arg((int)mdata->getModuleView()).arg(getModuleFile()));
+			mdata->setModuleView(MV_GENERIC);
+			break;
+	}
+
+	if(getSheetView())
+		((ModuleSV*)getSheetView())->reload();
+	if(getUserView())
+		((ModuleSV*)getUserView())->reload();
+	
+	m_moduleView = mdata->getModuleView();
+	switch(m_moduleView)
+	{
+		case MV_GENERIC:
+		{
+			// Calculate and set module size
+			mdata->setupGenericData();
+			
+			// set view size
+			if (getSheetView())
+			{
+				if (getSheetView()->isHidden())
+					getSheetView()->setHide(false);
+				emit getSheetView()->signalHide();
+				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getGenericSize()));
+			}
+			
+			if (getUserView())
+			{
+				if (!getUserView()->isHidden())
+					getUserView()->setHide(true);
+				emit getUserView()->signalHide();
+				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getGenericSize()));
+			}
+			
+			// Create Connectors
+			posList = mdata->getGenericConnPos();
+		}
+		break;
+		
+		case MV_PIXMAP:
+		{
+			// set view size
+			if (getSheetView())
+			{
+				if (getSheetView()->isHidden())
+					getSheetView()->setHide(false);
+				emit getSheetView()->signalHide();
+				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getPixmapSize()));
+			}
+			else
+			{
+				KSIMDEBUG_VAR("Module::reload MV_PIXMAP without SheetView", getModuleFile());
+			}
+			if (getUserView())
+			{
+				if (!getUserView()->isHidden())
+					getUserView()->setHide(true);
+				emit getUserView()->signalHide();
+				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getPixmapSize()));
+			}
+			
+			// Create Connectors
+			posList = mdata->getPixmapConnPos();
+		}
+		break;
+		
+		case MV_USERVIEW:
+		{
+			// set view size
+			if (getSheetView())
+			{
+				if (getSheetView()->isHidden())
+					getSheetView()->setHide(false);
+				emit getSheetView()->signalShow();
+				getSheetView()->setPlace(QRect(getSheetView()->getPos(), mdata->getUserViewSize()));
+			}
+			else
+			{
+				KSIMDEBUG_VAR("Module::reload MV_USERVIEW without SheetView", getModuleFile());
+			}
+			if (getUserView())
+			{
+				getUserView()->setPlace(QRect(getUserView()->getPos(), mdata->getUserViewSize()-QSize(2*gridX,2*gridY)));
+			}
+			else
+			{
+				KSIMDEBUG_VAR("Module::reload MV_USERVIEW without UserView", getModuleFile());
+			}
+			// Create Connectors
+			posList = mdata->getUserViewConnPos();
+		}
+		break;
+		
+		default:
+			KSIMDEBUG(QString::fromLatin1("Unknown module view = %1 Filename = %2").arg((int)mdata->getModuleView()).arg(getModuleFile()));
+			break;
+	}
+	
+	CHECK_PTR(posList);
+	for	(i=0; i < posList->count(); i++)
+	{
+		ExternalConnector * extConn = (ExternalConnector *)mdata->getExternalList()->at(i);
+		ConnectorBase *conn = searchConn (extConn, oldConnList);
+		
+		if (posList->at(i)->x() != -1)
+		{
+			if (conn)
+			{
+				// The connector exist
+				oldConnList->removeRef(conn);
+				addConnector(conn);
+			}
+			else
+			{
+				// The connector doesn't exist. Create it
+				conn = extConn->getExternalConn()->getConnInfo()
+				        ->create(this,
+				                 QString::fromLatin1("(extConn) %1").arg(extConn->getSerialNumber()),
+				                 extConn->getName(),
+				                 QPoint(0,0));
+			}
+
+			// Remove wire from external connector
+			if(extConn->getExternalConn()->isConnected())
+			{
+				extConn->getContainer()->delConnection(extConn->getExternalConn());
+			}
+
+			// Create optional conn if required
+			if(extConn->isOptionalConn())
+			{
+				OptionalConnector * oc;
+				oc = new OptionalConnector(conn, QString::fromLatin1("OptionalExtCon ")+conn->getName(),
+				                           conn->getName());
+				CHECK_PTR(oc);
+				oc->setEnabled(extConn->isOptionalConnEnabled() || conn->isConnected());
+				
+			}
+			extConn->setModuleConn(conn);
+			conn->setNegate(extConn->getExternalConn()->isNegated(),true);
+			
+			extList->append(extConn);
+			
+			conn->setGridPos(*(posList->at(i)));
+				
+			switch(mdata->getModuleView())
+			{
+				case MV_GENERIC:
+					conn->setOrientation(extConn->getExternalConn()->getOrientation());
+					break;
+				case MV_PIXMAP:
+					conn->setOrientation(*mdata->getPixmapConnOrient()->at(i));
+					break;
+				case MV_USERVIEW:
+					conn->setOrientation(*mdata->getUserViewConnOrient()->at(i));
+					break;
+				default:
+					break;
+			}
+		}
+		else
+		{
+			// Remove connection
+			if (conn && conn->getWire())
+			{
+				getContainer()->delConnection(conn);
+				oldConnList->removeRef(conn);
+				delete conn;
+			}
+		}
+	}
+	
+	// Remove unused connecetors
+	oldConnList->setAutoDelete(true);
+	while(oldConnList->count())
+	{
+		if (oldConnList->first()->getWire())
+			getContainer()->delConnection(oldConnList->first());
+		oldConnList->removeFirst();
+	}
+	delete oldConnList;
+}
+
+bool Module::checkRecursion() const
+{
+	CompContainer * parentContainer = getContainer();
+	
+	while(parentContainer && parentContainer->isParentComponent())
+	{
+		Component * comp = parentContainer->getParentComponent();
+		if (!comp)
+		{
+			KSIMDEBUG("parentContainer->getParentComponent() returns a null pointer!")
+			break;
+		}
+		
+		if (!comp->inherits("Module"))
+		{
+			KSIMDEBUG_VAR("Component should be a module but is not", comp->getName());
+			break;
+		}
+		
+		if (((Module *)comp)->getModuleFile() == getModuleFile())
+		{
+			// Same module file !!!!
+			return true;
+		}
+		parentContainer = comp->getContainer();
+	}
+	return false;
+}
+
+/** search the connector
+	returns 0,if no connector is found */
+ConnectorBase * Module::searchConn(const ExternalConnector * extConn, const ConnectorList * connList)
+{
+	const ConnectorInfo * extConnInfo = extConn->getExternalConn()->getConnInfo();
+	QString connWireName = QString::fromLatin1("(extConn) %1").arg(extConn->getSerialNumber());
+	FOR_EACH_CONNECTOR(it, *connList)
+	{
+		// Compare connector type and name
+		if ((it.current()->getConnInfo() == extConnInfo)
+			  &&(connWireName == it.current()->getWireName()))
+		{
+			// Both have same type and connector name have the correct wire name
+			return it.current();
+		}
+	}
+	return (ConnectorBase*)0;
+}
+
+
+/** Searches the external connector which is represented by the given connector.
+  * Returns 0 if no external connector will be found. */
+ExternalConnector * Module::searchExtConn(const ConnectorBase * conn) const
+{
+	for(unsigned int i = 0; i < getConnList()->count(); i++)
+	{
+		if (getConnList()->at(i) == conn)
+		{
+			return (ExternalConnector*) extList->at(i);
+		}
+	}
+	
+	return  (ExternalConnector*) 0;
+}
+
+
+/** Checks the component
+*   eg. all required inputs are connected
+*   The implementation calls the checks functions of the connectors and container.
+*	Returns the number of errors
+*/
+int Module::checkCircuit()
+{
+	int errors = Component::checkCircuit();
+	
+	errors += m_moduleContainer->checkCircuit();
+	
+	return errors;
+}
+
+/** Setup the Component for a new circuit execution.
+  * Calls the default implementation and the setup function of all containing components.
+  */
+void Module::setupCircuit()
+{
+	Component::setupCircuit();
+	
+	m_moduleContainer->setupCircuit();
+}
+
+/** Reset all simulation variables */
+void Module::reset()
+{
+	Component::reset();
+	// Call reset for all components
+	FOR_EACH_COMP(it,*m_moduleContainer->getComponentList())
+	{
+		it.current()->reset();
+	}
+}
+
+	
+/** Initialize the component popup menu */
+bool Module::initPopupMenu(QPopupMenu * popup)
+{
+	bool insert = Component::initPopupMenu(popup);
+	if (!insert)
+		popup->insertSeparator();
+	popup->insertItem(i18n("&Reload module"), this, SLOT(slotReload()));
+	return true;
+}
+
+/** Executes the reload function */
+void Module::slotReload()
+{
+	reloadModule();
+	refresh();
+}
+
+/** Setup the component lists for calculation, updateView.
+ 	* Call during simulation start. */
+void Module::setupSimulationList()
+{
+	getModuleContainer()->setupSimulationList();
 }
 
