@@ -29,6 +29,8 @@
 #include <qpixmap.h>
 
 // KDE-Includes
+#include <kmessagebox.h>
+#include <klocale.h>
 
 // Project-Includes
 #include "ksimdata.h"
@@ -46,6 +48,7 @@
 #include "externalconnector.h"
 #include "enumdict.h"
 #include "ksimdebug.h"
+#include "loglist.h"
 
 // Forward declaration
 
@@ -62,27 +65,34 @@ static const char * sPixmapFile 		= "PixFile";
 static const char * sModuleName 		= "Module Name";
 static const char * sModuleLibNames = "Module Lib Names";
 static const char * sShortDescr 		= "Short Descr";
-static const char * sViewAttribPixmap	= "View Attribute Pixmap";
-static const char * sViewAttribUserView	= "View Attribute User View";
 static const char * sPixmapStore	= "Pixmap Store";
 static const char * sPixmapData = "Pixmap Data";
 
-static EnumDict<ModuleViewType> moduleViewDict;
-
 EnumDict<ModuleViewType>::tData EnumDict<ModuleViewType>::data[]
-			= { {"Generic", MV_GENERIC},
+      = { {"Generic",    MV_GENERIC},
           {"User View",  MV_USERVIEW},
-					{"Pixmap", MV_PIXMAP},
-					{"None", MV_NONE},
-          {0,(ModuleViewType)0}};
+          {"Pixmap",     MV_PIXMAP},
+          {"None",       MV_NONE},
+          {0,           (ModuleViewType)0}};
+
+static const EnumDict<ModuleViewType> & getModuleViewDict()
+{
+	static EnumDict<ModuleViewType> moduleViewDict;
+	return moduleViewDict;
+}
 
-static EnumDict<ModulePixmapStoreType> modulePixmapStoreDict;
 
 EnumDict<ModulePixmapStoreType>::tData EnumDict<ModulePixmapStoreType>::data[]
-			= { {"Absolute", MPS_ABSOLTUE},
+      = { {"Absolute",         MPS_ABSOLTUE},
           {"Relative Module",  MPS_RELATIVE_MODULE},
-					{"Internal", MPS_INTERNAL},
-          {0,(ModulePixmapStoreType)0}};
+          {"Internal",         MPS_INTERNAL},
+          {0,                 (ModulePixmapStoreType)0}};
+
+static const EnumDict<ModulePixmapStoreType> & getModulePixmapStoreDict()
+{
+	static EnumDict<ModulePixmapStoreType> modulePixmapStoreDict;
+	return modulePixmapStoreDict;
+}
 
 
 
@@ -101,11 +111,9 @@ ModuleData::ModuleData(CompContainer * parent)
 		m_connOrientPixmap(0),
 		m_pixmap(0),
 		m_pixmapFile(),
-		m_viewAttribPixmap(VA_SHEETVIEW),
 		m_pixmapStore(MPS_ABSOLTUE),
 		m_connPosUserView(0),
 		m_connOrientUserView(0),
-		m_viewAttribUserView(VA_SHEETVIEW),
 		m_moduleView(MV_NONE),
 		m_container(parent),
 		m_moduleName(),
@@ -123,15 +131,10 @@ ModuleData::~ModuleData()
 	delete m_outList;
 	delete m_externalList;
 	
-	if (m_connPosGeneric)
-		delete m_connPosGeneric;
-	if (m_connPosPixmap)
-		delete m_connPosPixmap;
-	if (m_connOrientPixmap)
-		delete m_connOrientPixmap;
-		
-	if (m_pixmap)
-		delete m_pixmap;
+	delete m_connPosGeneric;
+	delete m_connPosPixmap;
+	delete m_connOrientPixmap;
+	delete m_pixmap;
 
 }
 	
@@ -212,11 +215,11 @@ const QString & ModuleData::getPixmapFile() const
 bool ModuleData::isPixmapFileValid() const
 {
 	return     (m_pixmapStore == MPS_INTERNAL)
-					|| (
-								 !m_pixmapFile.isEmpty()
-							&& (QFile::exists(m_pixmapFile))
-							&& (0 != QPixmap::imageFormat(m_pixmapFile))
-						 );
+	        || (
+	               !m_pixmapFile.isEmpty()
+	            && (QFile::exists(m_pixmapFile))
+	            && (0 != QPixmap::imageFormat(m_pixmapFile))
+	           );
 }
 
 /** Load the pixmap
@@ -226,8 +229,6 @@ bool ModuleData::loadPixmap()
 	if (m_pixmapStore == MPS_INTERNAL)
 		return false;
 	
-	
-	
 	bool res;
 	QPixmap * newPixmap = new QPixmap;
 	CHECK_PTR(newPixmap);
@@ -239,8 +240,9 @@ bool ModuleData::loadPixmap()
 		if (m_pixmap)
 			delete m_pixmap;
 		m_pixmap = newPixmap;
+		// Round up to next valid size and add connector space
 		m_pixmapSize = QSize( (((m_pixmap->width()  + 3*gridX - 1)/gridX)*gridX),
-													(((m_pixmap->height() + 3*gridY - 1)/gridY)*gridY) );
+		                      (((m_pixmap->height() + 3*gridY - 1)/gridY)*gridY) );
 	}
 	else
 	{
@@ -260,7 +262,7 @@ bool ModuleData::loadPixmap(const QString & filename)
 
 QSize ModuleData::getUserViewSize() const
 {
-	return m_container->getUserSize();
+	return m_container->getUserSize() + QSize(2*gridX, 2*gridY);
 }
 
 CompViewList * ModuleData::getUserViewList() const
@@ -292,27 +294,6 @@ OrientList * ModuleData::getUserViewConnOrient()
 };
 
 
-ViewAttribute ModuleData::getPixmapAttrib() const
-{
-	return m_viewAttribPixmap;
-}
-
-void ModuleData::setPixmapAttrib(ViewAttribute viewAttrib)
-{
-	m_viewAttribPixmap = viewAttrib;
-}
-
-ViewAttribute ModuleData::getUserViewAttrib() const
-{
-	return m_viewAttribUserView;
-}
-	
-void ModuleData::setUserViewAttrib(ViewAttribute viewAttrib)
-{
-	m_viewAttribUserView = viewAttrib;
-}
-
-
 /** Search externals in compList */
 void ModuleData::searchExternals(const ComponentList * compList)
 {
@@ -327,7 +308,7 @@ void ModuleData::searchExternals(const ComponentList * compList)
 	{
 		unsigned int i = 0;
 		QPoint a,b;
-    	
+		
 		if (it.current()->isExtConn())
 		{
 			ExternalConnector * extConn = (ExternalConnector *) it.current();
@@ -468,9 +449,9 @@ void ModuleData::setupPixmapData()
 		QPoint * pos = new QPoint(((ExternalConnector*)it.current())->getPixmapPos());
 		CHECK_PTR(pos);
 		m_connPosPixmap->append(pos);
-
+		
 		ConnOrientationType * orient = new ConnOrientationType(((ExternalConnector*)it.current())
-																			->getPixmapOrientation());
+		                                                          ->getPixmapOrientation());
 		CHECK_PTR(orient);
 		m_connOrientPixmap->append(orient);
 	}
@@ -510,9 +491,9 @@ void ModuleData::setupUserViewData()
 		QPoint * pos = new QPoint(((ExternalConnector*)it.current())->getUserViewPos());
 		CHECK_PTR(pos);
 		m_connPosUserView->append(pos);
-
+		
 		ConnOrientationType * orient = new ConnOrientationType(((ExternalConnector*)it.current())
-																					->getUserViewOrientation());
+		                                                          ->getUserViewOrientation());
 		CHECK_PTR(orient);
 		m_connOrientUserView->append(orient);
 	}
@@ -521,7 +502,7 @@ void ModuleData::setupUserViewData()
 /** save module properties */
 void ModuleData::save(KSimData & file)
 {
-	moduleViewDict.save(file, sViewType, m_moduleView);
+	getModuleViewDict().save(file, sViewType, m_moduleView);
 		
 	switch(m_pixmapStore)
 	{
@@ -538,20 +519,7 @@ void ModuleData::save(KSimData & file)
 			{
 				if(m_pixmap)
 				{
-					QByteArray array;
-					QDataStream stream(array, IO_WriteOnly);
-					QStringList strList;
-					unsigned int i;
-					
-					stream << *m_pixmap;
-					
-					strList.append(QString::number(array.size(),16));
-					
-					for (i = 0; i < array.size(); i++)
-					{
-						strList.append(QString::number((unsigned char)array[i],16));
-					}
-					file.writeEntry(sPixmapData, strList);
+					file.writeEntry(sPixmapData, *m_pixmap);
 				}
 				if(!m_pixmapFile.isEmpty())
 					file.writeEntry(sPixmapFile, m_pixmapFile);
@@ -562,7 +530,7 @@ void ModuleData::save(KSimData & file)
 			break;
 	}
 
-	modulePixmapStoreDict.save(file, sPixmapStore, m_pixmapStore);
+	getModulePixmapStoreDict().save(file, sPixmapStore, m_pixmapStore);
 	
 	if(!m_moduleName.isEmpty())
 		file.writeEntry(sModuleName, m_moduleName);
@@ -572,21 +540,15 @@ void ModuleData::save(KSimData & file)
 
 	if(!m_shortDescr.isEmpty())
 		file.writeEntry(sShortDescr, m_shortDescr);
-
-	if(m_viewAttribPixmap != VA_SHEETVIEW)
-		file.writeEntry(sViewAttribPixmap, ComponentInfo::convertViewAttribute(m_viewAttribPixmap));
-
-	if(m_viewAttribUserView != VA_SHEETVIEW)
-		file.writeEntry(sViewAttribUserView, ComponentInfo::convertViewAttribute(m_viewAttribUserView));
 }
 
 /** Load module properties
 	Returns true if successful */
 bool ModuleData::load(KSimData & file)
 {
-	m_moduleView = moduleViewDict.load(file, sViewType, MV_GENERIC);
+	m_moduleView = getModuleViewDict().load(file, sViewType, MV_GENERIC);
 	
-	m_pixmapStore = modulePixmapStoreDict.load(file, sPixmapStore, MPS_ABSOLTUE);
+	m_pixmapStore = getModulePixmapStoreDict().load(file, sPixmapStore, MPS_ABSOLTUE);
 	
 	switch(m_pixmapStore)
 	{
@@ -601,23 +563,15 @@ bool ModuleData::load(KSimData & file)
 		case MPS_INTERNAL:
 			{
 				m_pixmapFile = file.readEntry(sPixmapFile);
-				QStringList strList(file.readListEntry(sPixmapData));
-			
-				unsigned int size,i;
-				size = strList[0].toUInt(0,16);
-				QByteArray array(size);
-			
-				for (i = 0; i < size; i++)
-				{
-					array[i] = (char) strList[i+1].toUInt(0,16);
-				}
-			
-				QDataStream stream(array, IO_ReadOnly);
 				if (!m_pixmap)
+				{
 					m_pixmap = new QPixmap;
-				stream >> *m_pixmap;
+					CHECK_PTR(m_pixmap);
+				}
+				*m_pixmap = file.readPixmapEntry(sPixmapData);
+				// Round up to next valid size and add connector space
 				m_pixmapSize = QSize( (((m_pixmap->width()  + 3*gridX - 1)/gridX)*gridX),
-															(((m_pixmap->height() + 3*gridY - 1)/gridY)*gridY) );
+				                      (((m_pixmap->height() + 3*gridY - 1)/gridY)*gridY) );
 			}
 			break;
 		
@@ -632,23 +586,19 @@ bool ModuleData::load(KSimData & file)
 	m_moduleLibNames = file.readEntry(sModuleLibNames);
 	m_shortDescr = file.readEntry(sShortDescr);
 	
-	m_viewAttribPixmap = ComponentInfo::convertViewAttribute(file.readEntry(sViewAttribPixmap), VA_SHEETVIEW);
-	m_viewAttribUserView = ComponentInfo::convertViewAttribute(file.readEntry(sViewAttribUserView), VA_SHEETVIEW);
-
 	return true;
 }
-							
-							
+
 const ModuleInfo * ModuleData::makeModuleInfo(const QString & filename)
 {
-	KSimData file(filename);
+	KSimData file(filename, KSimData::versionAsIs, true);   // Read only
 	file.setGroup("/Property/Module/");
 	
-	ModuleViewType moduleView = moduleViewDict.load(file, sViewType, MV_NONE);
+	ModuleViewType moduleView = getModuleViewDict().load(file, sViewType, MV_NONE);
 	if(moduleView == MV_NONE)
 	{
 		// No module
-		return 0;
+		return (const ModuleInfo *)0;
 	}
 	
 	QString moduleName = file.readEntry(sModuleName);
@@ -667,18 +617,16 @@ const ModuleInfo * ModuleData::makeModuleInfo(const QString & filename)
 	{
 		case MV_NONE:
 		case MV_GENERIC:
+		case MV_PIXMAP:
 			break;
 		
 		case MV_USERVIEW:
-			viewAttrib = ComponentInfo::convertViewAttribute(file.readEntry(sViewAttribUserView), VA_SHEETVIEW);
-			break;
-	
-		case MV_PIXMAP:
-			viewAttrib = ComponentInfo::convertViewAttribute(file.readEntry(sViewAttribPixmap), VA_SHEETVIEW);
+			viewAttrib = VA_SHEET_AND_USER;
 			break;
 	
 		default:
 			KSIMDEBUG_VAR("Unknown module view", moduleView);
+			ASSERT(0);
 			break;
 	}	
 	
@@ -692,14 +640,14 @@ const ModuleInfo * ModuleData::makeModuleInfo(const QString & filename)
 	{
 		libName = moduleLibNameList[0];
 		moduleLibNameList.remove(libName);
-		addLibNames = moduleLibNameList.join(";");
+		addLibNames = moduleLibNameList.join(QString::fromLatin1(";"));
 	}
 	
 	return new ModuleInfo( filename,
-												 moduleName,
-												 libName,
-												 addLibNames,
-												 viewAttrib);
+	                       moduleName,
+	                       libName,
+	                       addLibNames,
+	                       viewAttrib);
 
 }
 
@@ -710,7 +658,7 @@ KURL ModuleData::getUrl() const
 	{
 		// Module data is part of document
 		const KURL & url = m_container->getDoc()->URL();
-		KSIMDEBUG_VAR("Document Path",url.path());
+//		KSIMDEBUG_VAR("Document Path",url.path());
 		return url;
 	}
 	else
@@ -718,7 +666,7 @@ KURL ModuleData::getUrl() const
 		// Module data is part of a module
 		Module * module = (Module*) m_container->getParentComponent();
 		KURL url(module->getModuleFile());
-		KSIMDEBUG_VAR("Module Path",url.path());
+//		KSIMDEBUG_VAR("Module Path",url.path());
 		return url;
 	}
 }
@@ -758,7 +706,7 @@ QString ModuleData::getRelativePath()
 	if(valid)
 	{
 		QString result(KSimFile::relativePath(m_pixmapFile,fileInfo.dirPath(true)));
-		KSIMDEBUG_VAR("Relative file name",result);
+//		KSIMDEBUG_VAR("Relative file name",result);
 		
 		return result;
 	}
@@ -776,7 +724,7 @@ void ModuleData::setRelativePath(QString relPath)
 	KURL url(getUrl());
 	QString path(url.path());
 	QFileInfo fileInfo(path);
-	QDir dir(fileInfo.dirPath(true));	
+//	QDir dir(fileInfo.dirPath(true));	
 	
 	bool valid(true);
 
@@ -806,9 +754,12 @@ void ModuleData::setRelativePath(QString relPath)
 	
 	if(valid)
 	{
-		KSIMDEBUG_VAR("File path",dir.path());
+		KSIMDEBUG_VAR("File path",url.directory(false, true));
 		KSIMDEBUG_VAR("Pixmap relative file",relPath);
-		int lastSlash(relPath.find('/',-1));
+		QFileInfo fi(url.directory(false, true) + relPath);
+		m_pixmapFile = fi.absFilePath();
+		
+/*		int lastSlash(relPath.find('/',-1));
 		if(-1 != lastSlash)
 		{
 			// relPath contains directories
@@ -819,7 +770,7 @@ void ModuleData::setRelativePath(QString relPath)
 		{
 			// relPath contains no directories
 			m_pixmapFile = dir.absPath() + '/' + relPath.right(lastSlash);
-		}
+		}*/
 		
 		KSIMDEBUG_VAR("Pixmap file",m_pixmapFile);
 	}
@@ -831,5 +782,84 @@ void ModuleData::setRelativePath(QString relPath)
 }
 
 
+bool ModuleData::checkConnectorPosition(bool showMessage)
+{
+	bool matchPos(true);
+	const PointList * connPosList = (PointList *)0;
+	QSize size;
+
+	switch(getModuleView())
+	{
+		case MV_NONE:
+		case MV_GENERIC:
+			// Nothing yet
+			return matchPos;
+			break;
+
+		case MV_PIXMAP:
+			connPosList = getPixmapConnPos();
+			size = getPixmapSize();
+			break;
+
+		case MV_USERVIEW:
+			connPosList = getUserViewConnPos();
+			size = getUserViewSize();
+			break;
+
+		default:
+			KSIMDEBUG(QString::fromLatin1("Unknown module view = %1").arg((int)getModuleView()));
+			return matchPos;
+			break;
+	}
+
+	size.rwidth()  /= gridX;
+	size.rheight() /= gridY;
+	QRect topArea   (             1,               0, size.width()-2,               1);
+	QRect bottomArea(             1, size.height()-1, size.width()-2,               1);
+	QRect leftArea  (             0,               1,              1, size.height()-2);
+	QRect rightArea (size.width()-1,               1,              1, size.height()-2);
+
+	for (QListIterator<QPoint> it(*connPosList);it.current();++it)
+	{
+		// not hidden?
+		if (it.current()->x() != -1)
+		{
+			// Connector inside one area
+			if ( topArea.contains(*it.current())
+			  || bottomArea.contains(*it.current())
+			  || leftArea.contains(*it.current())
+			  || rightArea.contains(*it.current()) )
+			{
+				// Okay
+			}
+			else
+			{
+				KSIMDEBUG(QString::fromLatin1("Failed (%1/%2)").arg(it.current()->x()).arg(it.current()->y()));
+				KSIMDEBUG(QString::fromLatin1("Size (%1/%2)").arg(size.width()).arg(size.height()));
+				KSIMDEBUG(QString::fromLatin1("topArea (%1/%2 - %3/%4)").arg(topArea.left()).arg(topArea.top()).arg(topArea.right()).arg(topArea.bottom()));
+				KSIMDEBUG(QString::fromLatin1("bottomArea (%1/%2 - %3/%4)").arg(bottomArea.left()).arg(bottomArea.top()).arg(bottomArea.right()).arg(bottomArea.bottom()));
+				KSIMDEBUG(QString::fromLatin1("leftArea (%1/%2 - %3/%4)").arg(leftArea.left()).arg(leftArea.top()).arg(leftArea.right()).arg(leftArea.bottom()));
+				KSIMDEBUG(QString::fromLatin1("rightArea (%1/%2 - %3/%4)").arg(rightArea.left()).arg(rightArea.top()).arg(rightArea.right()).arg(rightArea.bottom()));
+				matchPos = false;
+//				break;
+			}
+		}
+	}
+
+	if (!matchPos && showMessage)
+	{
+		QString s(i18n("The placement of the external connectors don't match the module shape! Switch to generic view.\n"
+		               "Hint: The common reason for this error is a resized module without rearranged external connectors."));
+		if (m_container->isParentComponent())
+		{
+			m_container->getParentComponent()->logError(s);
+		}
+		else
+		{
+			m_container->getLogList()->error(s);
+		}
+	}
+	return matchPos;
+}
 
 
