@@ -23,6 +23,7 @@
 #include <qlineedit.h>
 #include <qpopupmenu.h>
 #include <qintdict.h>
+#include <qcursor.h>
 
 // KDE-Includes
 #include <klocale.h>
@@ -39,17 +40,16 @@
 //#############################################################
 //#############################################################
 
-class KSimDoubleSpinBoxPrivate
+class KSimDoubleSpinBox::Private
 {
-	friend class KSimDoubleSpinBox;
-	
-	KSimDoubleSpinBoxPrivate(double value, double min, double max, double steps)
+public:
+	Private(double value, double min, double max, double steps)
 		:	m_value(value),
 			m_min(min),
 			m_max(max),
 			m_step(steps),
 			m_expEna(false),
-			m_expSteps(0),
+			m_expSteps((KSimExponentStep *)0L),
 			m_fieldwidth(0),
 			m_format('g'),
 			m_precision(-1),
@@ -60,10 +60,9 @@ class KSimDoubleSpinBoxPrivate
 	{
 	};
 	
-	~KSimDoubleSpinBoxPrivate()
+	~Private()
 	{
-		if (m_expSteps)
-			delete m_expSteps;
+		delete m_expSteps;
 	};
 		
 	double value() const { return m_value; };
@@ -114,12 +113,12 @@ class KSimDoubleSpinBoxPrivate
 	static int rmbExponentStepID;
 };
 
-int KSimDoubleSpinBoxPrivate::rmbExponentStepID = -1;
+int KSimDoubleSpinBox::Private::rmbExponentStepID = -1;
 
-#define QSPINBOX_UPPER_LIMIT	2
-#define QSPINBOX_TOGGLE_1			1
-#define QSPINBOX_TOGGLE_0			0
-#define QSPINBOX_LOWER_LIMIT	-1
+#define QSPINBOX_UPPER_LIMIT  2
+#define QSPINBOX_TOGGLE_1     1
+#define QSPINBOX_TOGGLE_0     0
+#define QSPINBOX_LOWER_LIMIT -1
 
 
 
@@ -127,7 +126,7 @@ int KSimDoubleSpinBoxPrivate::rmbExponentStepID = -1;
 
 KSimDoubleSpinBox::KSimDoubleSpinBox(QWidget * parent, const char * name)
 	: QSpinBox(parent, name),
-		m_p(new KSimDoubleSpinBoxPrivate(0.0, -10.0, 10.0, 0.1))
+		m_p(new Private(0.0, -10.0, 10.0, 0.1))
 {
 	init();
 }
@@ -135,7 +134,7 @@ KSimDoubleSpinBox::KSimDoubleSpinBox(QWidget * parent, const char * name)
 KSimDoubleSpinBox::KSimDoubleSpinBox(double value, double min, double max, double steps,
                                      QWidget * parent, const char * name)
 	: QSpinBox(parent, name),
-		m_p(new KSimDoubleSpinBoxPrivate(value, min, max, steps))
+		m_p(new Private(value, min, max, steps))
 {
 	init();
 }
@@ -144,10 +143,10 @@ void KSimDoubleSpinBox::init()
 {
 	CHECK_PTR(m_p);
 	setValidator(0);
-  connect(this,SIGNAL(valueChanged(int)), this, SLOT(slotValueChanged(int)));
-  QSpinBox::setMinValue(QSPINBOX_LOWER_LIMIT);
-  QSpinBox::setMaxValue(QSPINBOX_UPPER_LIMIT);
-  setFocusPolicy(QWidget::WheelFocus);
+	connect(this,SIGNAL(valueChanged(int)), this, SLOT(slotValueChanged(int)));
+	QSpinBox::setMinValue(QSPINBOX_LOWER_LIMIT);
+	QSpinBox::setMaxValue(QSPINBOX_UPPER_LIMIT);
+	setFocusPolicy(QWidget::WheelFocus);
 	setAlignment(AlignRight);
 }
 
@@ -388,7 +387,7 @@ void KSimDoubleSpinBox::stepDown()
 
 QString KSimDoubleSpinBox::mapValueToText(int)
 {
-	return QString("%1").arg(value(),fieldwidth(),format(),precision());
+	return QString::fromLatin1("%1").arg(value(),fieldwidth(),format(),precision());
 }
 
 int KSimDoubleSpinBox::mapTextToValue(bool *)
@@ -623,18 +622,18 @@ void KSimDoubleSpinBox::initRmbMenu(QPopupMenu * popup)
 {
 	if (exponentStepPossible())
 	{
-		KSimDoubleSpinBoxPrivate::rmbExponentStepID = popup->insertItem(i18n("Enable exponent steps"));
-		popup->setItemChecked(KSimDoubleSpinBoxPrivate::rmbExponentStepID, exponentStepEnabled());
+		Private::rmbExponentStepID = popup->insertItem(i18n("Enable exponent steps"));
+		popup->setItemChecked(Private::rmbExponentStepID, exponentStepEnabled());
 	}
 	else
 	{
-		KSimDoubleSpinBoxPrivate::rmbExponentStepID = -1; // invalid value
+		Private::rmbExponentStepID = -1; // invalid value
 	}
 }
 
 void KSimDoubleSpinBox::execRmbMenu(int ID)
 {
-	if (ID == KSimDoubleSpinBoxPrivate::rmbExponentStepID)
+	if (ID == Private::rmbExponentStepID)
 	{
 		setExponentStepEnabled(!exponentStepEnabled());
 	}
@@ -643,20 +642,20 @@ void KSimDoubleSpinBox::execRmbMenu(int ID)
 //#############################################################
 //#############################################################
 
-class KSimDoubleUnitSpinBoxPrivate
+class KSimDoubleUnitSpinBox::Private
 {
-	friend class KSimDoubleUnitSpinBox;
+public:
 
-	KSimDoubleUnitSpinBoxPrivate()
+	Private()
 		: currentUnit(0),
 			lastUnit(0),
 			fixedUnit(0),
-			currentUnitList("")
+			currentUnitList(QString::fromLatin1(""))
 	{
 		multiUnitList = new KSimMultiUnitList();
 		CHECK_PTR(multiUnitList);
 	};
-	~KSimDoubleUnitSpinBoxPrivate()
+	~Private()
 	{
 		delete multiUnitList;
 	};
@@ -672,23 +671,23 @@ class KSimDoubleUnitSpinBoxPrivate
 	static QIntDict<QString> * rmbUnitIDList;
 };
 
-int KSimDoubleUnitSpinBoxPrivate::rmbAutoUnitID = -1;
-QIntDict<QString> * KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList = 0;
+int KSimDoubleUnitSpinBox::Private::rmbAutoUnitID = -1;
+QIntDict<QString> * KSimDoubleUnitSpinBox::Private::rmbUnitIDList = (QIntDict<QString> *)0L;
 
 //#############################################################
 //#############################################################
 
 KSimDoubleUnitSpinBox::KSimDoubleUnitSpinBox(QWidget * parent, const char * name)
 	:	KSimDoubleSpinBox(parent, name),
-		m_p(new KSimDoubleUnitSpinBoxPrivate())
+		m_p(new Private())
 {
 	CHECK_PTR(m_p);
 }
 
 KSimDoubleUnitSpinBox::KSimDoubleUnitSpinBox(double min, double max, double steps,
-                        										QWidget * parent, const char * name)
+                                             QWidget * parent, const char * name)
 	:	KSimDoubleSpinBox(0.0, min, max, steps, parent, name),
-		m_p(new KSimDoubleUnitSpinBoxPrivate())
+		m_p(new Private())
 {
 	CHECK_PTR(m_p);
 }
@@ -715,15 +714,15 @@ int KSimDoubleUnitSpinBox::setValueInternal(double newValue)
 		const KSimUnitBase * unit;
 		
 		res = KSimDoubleSpinBox::setValueInternal(newValue);
-
-    // Find prefered Unit
-    if ((m_p->currentUnit && m_p->currentUnit->isPrefered(value())) || (m_p->fixedUnit))
-    {
-    	unit = m_p->currentUnit;
+		
+		// Find prefered Unit
+		if ((m_p->currentUnit && m_p->currentUnit->isPrefered(value())) || (m_p->fixedUnit))
+		{
+			unit = m_p->currentUnit;
 //			KSIMDEBUG("unit = m_p->currentUnit");
-    }
-    else
-    {
+		}
+		else
+		{
 //			KSIMDEBUG_VAR("",m_p->currentUnitList);
 			unit = getMultiUnitList().findPrefered(value(), &m_p->currentUnitList);
 //			KSIMDEBUG("unit = getMultiUnitList().findPrefered(value(), &m_p->currentUnitList)");
@@ -733,13 +732,13 @@ int KSimDoubleUnitSpinBox::setValueInternal(double newValue)
 //			KSIMDEBUG_VAR("", value());
 //			KSIMDEBUG_VAR("", unit->getUnitString());
 			m_p->currentUnit = unit;
-			setSuffix(QString(" ") + m_p->currentUnit->getUnitString());
+			setSuffix(QString::fromLatin1(" ") + m_p->currentUnit->getUnitString());
 		}
 		else
 		{
 			// No unit found
 			KSIMDEBUG("No prefered unit found.");
-			setSuffix(QString(" "));
+			setSuffix(QString::fromLatin1(" "));
 		}
 		m_p->lastUnit = unit;
 	}
@@ -758,7 +757,7 @@ QString KSimDoubleUnitSpinBox::mapValueToText(int newValue)
 	{
 //		KSIMDEBUG_VAR("mapValueToText",value());
 //		KSIMDEBUG_VAR("mapValueToText",m_p->currentUnit->toUnit(value()));
-		return QString("%1").arg(m_p->currentUnit->toUnit(value()),fieldwidth(),format(),precision());
+		return QString::fromLatin1("%1").arg(m_p->currentUnit->toUnit(value()),fieldwidth(),format(),precision());
 	}
 	return KSimDoubleSpinBox::mapValueToText(newValue);
 }
@@ -907,13 +906,13 @@ void KSimDoubleUnitSpinBox::initRmbMenu(QPopupMenu * popup)
 		popup->insertSeparator();
 	}
 	
-	KSimDoubleUnitSpinBoxPrivate::rmbAutoUnitID = popup->insertItem(i18n("Auto unit"));
-	popup->setItemChecked(KSimDoubleUnitSpinBoxPrivate::rmbAutoUnitID, fixedUnit() == 0);
+	KSimDoubleUnitSpinBox::Private::rmbAutoUnitID = popup->insertItem(i18n("Auto unit"));
+	popup->setItemChecked(KSimDoubleUnitSpinBox::Private::rmbAutoUnitID, fixedUnit() == 0);
 	
 	//####  Add units
 	
-	KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList = new QIntDict<QString>;
-	CHECK_PTR(KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList);
+	KSimDoubleUnitSpinBox::Private::rmbUnitIDList = new QIntDict<QString>;
+	CHECK_PTR(KSimDoubleUnitSpinBox::Private::rmbUnitIDList);
 	
 	QListIterator<KSimUnitList> mulitListIt(getMultiUnitList().getList());
 	
@@ -936,17 +935,17 @@ void KSimDoubleUnitSpinBox::initRmbMenu(QPopupMenu * popup)
 						break;
 					}
 				}
-	    	if (unitIt.current())
-  	  	{
-	  	  	// Insert unit
-	  	  	int ID = popup->insertItem(unitIt.current()->getUnitString());
-  	  		KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList->insert(ID, &unitIt.current()->getUnitString());
-  	  	}
-  	  	break;
-  	  }
-  	  	
-  	  default:
-  	  {
+				if (unitIt.current())
+				{
+					// Insert unit
+					int ID = popup->insertItem(unitIt.current()->getUnitString());
+					KSimDoubleUnitSpinBox::Private::rmbUnitIDList->insert(ID, &unitIt.current()->getUnitString());
+				}
+				break;
+			}
+
+			default:
+			{
 				// Create own popup
 				QPopupMenu * unitPopup = new QPopupMenu (popup, mulitListIt.current()->getListName());
 				CHECK_PTR(unitPopup);
@@ -954,27 +953,27 @@ void KSimDoubleUnitSpinBox::initRmbMenu(QPopupMenu * popup)
 				QListIterator<KSimUnitBase> unitIt(*mulitListIt.current());
 				for (; unitIt.current(); ++unitIt)
 				{
-    			if (!unitIt.current()->isHidden())
-    			{
-	    			int ID = unitPopup->insertItem(unitIt.current()->getUnitString());
-  	  			KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList->insert(ID, &unitIt.current()->getUnitString());
-  	  		}
-    		}
+					if (!unitIt.current()->isHidden())
+					{
+						int ID = unitPopup->insertItem(unitIt.current()->getUnitString());
+						KSimDoubleUnitSpinBox::Private::rmbUnitIDList->insert(ID, &unitIt.current()->getUnitString());
+					}
+				}
 				popup->insertItem(mulitListIt.current()->getListName(), unitPopup);
 				break;
 			}
-		}		
+		}
 	}
 }
 
 void KSimDoubleUnitSpinBox::execRmbMenu(int ID)
 {
 	KSimDoubleSpinBox::execRmbMenu(ID);
-	if (ID == KSimDoubleUnitSpinBoxPrivate::rmbAutoUnitID)
+	if (ID == KSimDoubleUnitSpinBox::Private::rmbAutoUnitID)
 	{
 		if (fixedUnit())
 		{
-			setFixedUnit("");
+			setFixedUnit(QString::fromLatin1(""));
 		}
 		else
 		{
@@ -983,17 +982,17 @@ void KSimDoubleUnitSpinBox::execRmbMenu(int ID)
 	}
 	else
 	{
-		if (KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList)
+		if (KSimDoubleUnitSpinBox::Private::rmbUnitIDList)
 		{
-			QString * unitStr = KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList->find(ID);
+			QString * unitStr = KSimDoubleUnitSpinBox::Private::rmbUnitIDList->find(ID);
 			if (unitStr)
 			{
 				setFixedUnit(*unitStr);
 			}
 		}
 	}
-	delete KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList;
-	KSimDoubleUnitSpinBoxPrivate::rmbUnitIDList = 0;
+	delete KSimDoubleUnitSpinBox::Private::rmbUnitIDList;
+	KSimDoubleUnitSpinBox::Private::rmbUnitIDList = (QIntDict<QString> *)0L;
 }
 
 
